# The GHRD create_ghrd_qsys.tcl will call each of those subsystem construct script
# automatically based on the corresponding parameter argument defined
# example command to execute this script file
#   qsys-script --script=construct_subsys_peripheral.tcl --cmd="set subsys_name qsys_periph"

source ./arguments_solver.tcl
source ./utils.tcl
set subsys_name subsys_sgmii_adapter
  
package require -exact qsys 18.1

create_system $subsys_name

set_project_property DEVICE_FAMILY $device_family
set_project_property DEVICE $device
set_validation_property AUTOMATIC_VALIDATION false

add_component_param "altera_clock_bridge clk_bdg
					IP_FILE_PATH ip/$subsys_name/clk_bdg.ip 
					NUM_CLOCK_OUTPUTS 1
					"

add_component_param "altera_reset_bridge rst_bdg
					IP_FILE_PATH ip/$subsys_name/rst_bdg.ip 
					ACTIVE_LOW_RESET 1
					SYNCHRONOUS_EDGES both
					NUM_RESET_OUTPUTS 1
					USE_RESET_REQUEST 0
					"

add_component_param "altera_avalon_mm_bridge pb 
					IP_FILE_PATH ip/$subsys_name/pb.ip 
					DATA_WIDTH 32
					ADDRESS_WIDTH 7
					MAX_BURST_SIZE 1
					MAX_PENDING_RESPONSES 1
					"							
add_component_param "altera_hps_emac_interface_splitter emac_splitter
                    IP_FILE_PATH ip/$subsys_name/emac_splitter.ip
                    "

add_component_param "altera_gmii_to_sgmii_adapter gmii_to_sgmii_adapter
                    IP_FILE_PATH ip/$subsys_name/gmii_to_sgmii_adapter.ip
				    "

add_component_param "altera_eth_tse tse_phy
                    IP_FILE_PATH ip/$subsys_name/tse_phy.ip
                    core_variation PCS_ONLY
                    ifGMII MII_GMII
                    transceiver_type LVDS_IO
                    enable_sgmii true
				    "

# connections and connection parameters
connect_map "pb.m0 gmii_to_sgmii_adapter.avalon_slave 0x040"
connect_map "pb.m0 tse_phy.control_port 0x0"

connect "clk_bdg.out_clk rst_bdg.clk
         clk_bdg.out_clk pb.clk
         clk_bdg.out_clk gmii_to_sgmii_adapter.peri_clock
         clk_bdg.out_clk tse_phy.control_port_clock_connection
         "

connect "rst_bdg.out_reset pb.reset
         rst_bdg.out_reset gmii_to_sgmii_adapter.peri_reset
         rst_bdg.out_reset tse_phy.reset_connection
         "

connect "gmii_to_sgmii_adapter.hps_gmii emac_splitter.hps_gmii
         gmii_to_sgmii_adapter.pcs_clock_enable tse_phy.clock_enable_connection 
         gmii_to_sgmii_adapter.pcs_gmii tse_phy.gmii_connection
         gmii_to_sgmii_adapter.pcs_mii tse_phy.mii_connection 
         tse_phy.pcs_receive_clock_connection gmii_to_sgmii_adapter.pcs_receive_clock
         gmii_to_sgmii_adapter.pcs_receive_reset tse_phy.pcs_receive_reset_connection
         tse_phy.pcs_transmit_clock_connection gmii_to_sgmii_adapter.pcs_transmit_clock 
         gmii_to_sgmii_adapter.pcs_transmit_reset tse_phy.pcs_transmit_reset_connection
         "

# exported interfaces
export clk_bdg in_clk clk
export rst_bdg in_reset reset
export pb s0 pb_s
export emac_splitter emac emac
export emac_splitter emac_gtx_clk emac_gtx_clk
export emac_splitter emac_rx_clk_in emac_rx_clk_in
export emac_splitter emac_rx_reset emac_rx_reset
export emac_splitter emac_tx_clk_in emac_tx_clk_in
export emac_splitter emac_tx_reset emac_tx_reset
export emac_splitter mdio mdio
export emac_splitter ptp ptp
export tse_phy pcs_ref_clk_clock_connection pcs_refclk
export tse_phy serdes_control_connection serdes_control
export tse_phy serial_connection serial
export tse_phy sgmii_status_connection sgmii_status
export tse_phy status_led_connection status_led

# interconnect requirements
set_interconnect_requirement {$system} {qsys_mm.clockCrossingAdapter} {AUTO}
set_interconnect_requirement {$system} {qsys_mm.maxAdditionalLatency} {1}
set_interconnect_requirement {$system} {qsys_mm.enableEccProtection} {FALSE}
set_interconnect_requirement {$system} {qsys_mm.insertDefaultSlave} {FALSE}
	
sync_sysinfo_parameters	
	
save_system ${subsys_name}.qsys
