// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on

module altera_pcie_s10_dma_controller #(
	  parameter                                    DEVICE_FAMILY = "Stratix 10",
	  parameter                                    DMA_WIDTH          = 256
  ) (
      input logic                                  Clk_i,
      input logic                                  Rstn_i,

      input logic   [81:0]                         MsiInterface_i,

//// DMA Read Interface
      // AVMM Register Slave Port (Write only)
      input  logic                                  RdDCSChipSelect_i,
      input  logic                                  RdDCSWrite_i,
      input  logic                                  RdDCSRead_i,
      input  logic  [7:0]                           RdDCSAddress_i,
      input  logic  [31:0]                          RdDCSWriteData_i,
      output  logic [31:0]                          RdDCSReadData_o,
      input  logic  [3:0]                           RdDCSByteEnable_i,
      output logic                                  RdDCSWaitRequest_o,


            // AVMM Register Master Port (Write only)

      output   logic [63:0]                         RdDCMAddress_o,
      output                                        RdDCMWrite_o,
      output   logic [31:0]                         RdDCMWriteData_o,
      output                                        RdDCMRead_o,
      output   logic [3:0]                          RdDCMByteEnable_o,
      input    logic                                RdDCMWaitRequest_i,
      input    logic [31:0]                         RdDCMReadData_i,
      input    logic                                RdDCMReadDataValid_i,

      /// DT 256-bit slave interface (Write only)

      input  logic                                  RdDTSChipSelect_i,
      input  logic                                  RdDTSWrite_i,
      input  logic  [4:0]                           RdDTSBurstCount_i,
      input  logic  [7:0]                           RdDTSAddress_i,
      input  logic  [DMA_WIDTH-1:0]                 RdDTSWriteData_i,
      output logic                                  RdDTSWaitRequest_o,

      /// DMA programming interface
      output   logic  [159:0]                       RdDmaTxData_o,
      output   logic                                RdDmaTxValid_o,
      input    logic                                RdDmaTxReady_i,

     // DMA Status Interface
      input   logic  [31:0]                         RdDmaRxData_i,
      input   logic                                 RdDmaRxValid_i,

////                DMA Write Interface              ///////////////////////////
 //////////      // AVMM Register Slave Port (Write only)   /////////////////////
      input  logic                                  WrDCSChipSelect_i,
      input  logic                                  WrDCSWrite_i,
      input  logic                                  WrDCSRead_i,
      input  logic  [7:0]                           WrDCSAddress_i,
      input  logic  [31:0]                          WrDCSWriteData_i,
      output logic  [31:0]                          WrDCSReadData_o,
      input  logic  [3:0]                           WrDCSByteEnable_i,
      output logic                                  WrDCSWaitRequest_o,


            // AVMM Register Master Port (Write only)

      output   logic [63:0]                         WrDCMAddress_o,
      output                                        WrDCMWrite_o,
      output   logic [31:0]                         WrDCMWriteData_o,
      output                                        WrDCMRead_o,
      output   logic [3:0]                          WrDCMByteEnable_o,
      input    logic                                WrDCMWaitRequest_i,
      input    logic [31:0]                         WrDCMReadData_i,
      input    logic                                WrDCMReadDataValid_i,

      /// DT 256-bit slave interface (Write only)

      input  logic                                  WrDTSChipSelect_i,
      input  logic                                  WrDTSWrite_i,
      input  logic  [4:0]                           WrDTSBurstCount_i,
      input  logic  [7:0]                           WrDTSAddress_i,
      input  logic  [DMA_WIDTH-1:0]                 WrDTSWriteData_i,
      output logic                                  WrDTSWaitRequest_o,

      /// DMA programming interface
      output   logic  [159:0]                       WrDmaTxData_o,
      output   logic                                WrDmaTxValid_o,
      input    logic                                WrDmaTxReady_i,

     // DMA Status Interface
      input   logic  [31:0]                         WrDmaRxData_i,
      input   logic                                 WrDmaRxValid_i
);


      /// DT programming interface
logic  [159:0]       wrdt_dmatxdata ,rddt_dmatxdata  ,rddsc_dmatxdata   ;
logic                wrdt_dmatxvalid,rddt_dmatxvalid ,rddsc_dmatxvalid  ;
logic                wrdt_dmatxready,rddt_dmatxready ,rddsc_dmatxready  ;

	  
data_mover_control #(
     .DESC_ID           (8'h80),
     .DEVICE_FAMILY     (DEVICE_FAMILY) 	 
  ) 
  rd_control(
      .Clk_i                      (Clk_i             ),
      .Rstn_i                     (Rstn_i            ),
      .MsiInterface_i             (MsiInterface_i    ),
	   //////////      // AVMM Register Slave Port (Write only)   
      .DCSChipSelect_i            (RdDCSChipSelect_i   ),
      .DCSWrite_i                 (RdDCSWrite_i        ),
      .DCSRead_i                  (RdDCSRead_i         ),
      .DCSAddress_i               (RdDCSAddress_i      ),
      .DCSWriteData_i             (RdDCSWriteData_i    ),
      .DCSReadData_o              (RdDCSReadData_o     ),
      .DCSByteEnable_i            (RdDCSByteEnable_i   ),
      .DCSWaitRequest_o           (RdDCSWaitRequest_o  ),
	   //////////      // AVMM Status Update Master Port (Write only)  
      .DCMAddress_o               (RdDCMAddress_o      ),	  
      .DCMWrite_o                 (RdDCMWrite_o        ),
      .DCMWriteData_o             (RdDCMWriteData_o    ),
      .DCMRead_o                  (RdDCMRead_o         ),
      .DCMByteEnable_o            (RdDCMByteEnable_o   ),
      .DCMWaitRequest_i           (RdDCMWaitRequest_i  ),
      .DCMReadData_i              (RdDCMReadData_i     ),
      .DCMReadDataValid_i         (RdDCMReadDataValid_i),
      /// DT 256-bit slave interface (Write only)	  
      .DTSChipSelect_i            (RdDTSChipSelect_i   ),
      .DTSWrite_i                 (RdDTSWrite_i        ),
      .DTSBurstCount_i            (RdDTSBurstCount_i   ),
      .DTSAddress_i               (RdDTSAddress_i      ),
      .DTSWriteData_i             (RdDTSWriteData_i    ),
      .DTSWaitRequest_o           (RdDTSWaitRequest_o  ),
	  
      .DT_DmaTxData_o             (rddt_dmatxdata      ),
      .DT_DmaTxValid_o            (rddt_dmatxvalid     ),
      .DT_DmaTxReady_i            (rddt_dmatxready     ),
      .Desc_DmaTxData_o           (rddsc_dmatxdata     ),
      .Desc_DmaTxValid_o          (rddsc_dmatxvalid    ),
      .Desc_DmaTxReady_i          (rddsc_dmatxready    ),
	  
      .DmaRxData_i                (RdDmaRxData_i       ),
      .DmaRxValid_i               (RdDmaRxValid_i      )
);

data_mover_control #(
     .DESC_ID           (8'h81),
     .DEVICE_FAMILY     (DEVICE_FAMILY) 	 
  ) 
 wr_control(
      .Clk_i                      (Clk_i             ),
      .Rstn_i                     (Rstn_i            ),
      .MsiInterface_i             (MsiInterface_i    ),
	   //////////      // AVMM Register Slave Port (Write only)   
      .DCSChipSelect_i            (WrDCSChipSelect_i   ),
      .DCSWrite_i                 (WrDCSWrite_i        ),
      .DCSRead_i                  (WrDCSRead_i         ),
      .DCSAddress_i               (WrDCSAddress_i      ),
      .DCSWriteData_i             (WrDCSWriteData_i    ),
      .DCSReadData_o              (WrDCSReadData_o     ),
      .DCSByteEnable_i            (WrDCSByteEnable_i   ),
      .DCSWaitRequest_o           (WrDCSWaitRequest_o  ),
	   //////////      // AVMM Status Update Master Port (Write only)  
      .DCMAddress_o               (WrDCMAddress_o      ),	  
      .DCMWrite_o                 (WrDCMWrite_o        ),
      .DCMWriteData_o             (WrDCMWriteData_o    ),
      .DCMRead_o                  (WrDCMRead_o         ),
      .DCMByteEnable_o            (WrDCMByteEnable_o   ),
      .DCMWaitRequest_i           (WrDCMWaitRequest_i  ),
      .DCMReadData_i              (WrDCMReadData_i     ),
      .DCMReadDataValid_i         (WrDCMReadDataValid_i),
      /// DT 256-bit slave interface (Write only)	  
      .DTSChipSelect_i            (WrDTSChipSelect_i   ),
      .DTSWrite_i                 (WrDTSWrite_i        ),
      .DTSBurstCount_i            (WrDTSBurstCount_i   ),
      .DTSAddress_i               (WrDTSAddress_i      ),
      .DTSWriteData_i             (WrDTSWriteData_i    ),
      .DTSWaitRequest_o           (WrDTSWaitRequest_o  ),
	  
      .DT_DmaTxData_o             (wrdt_dmatxdata      ),
      .DT_DmaTxValid_o            (wrdt_dmatxvalid     ),
      .DT_DmaTxReady_i            (wrdt_dmatxready     ),
      .Desc_DmaTxData_o           (WrDmaTxData_o       ),
      .Desc_DmaTxValid_o          (WrDmaTxValid_o      ),
      .Desc_DmaTxReady_i          (WrDmaTxReady_i      ),
	  
      .DmaRxData_i                (WrDmaRxData_i       ),
      .DmaRxValid_i               (WrDmaRxValid_i      )
);

rr_arbiter arbiter
 (
      .Clk_i                      (Clk_i               ),
      .Rstn_i                     (Rstn_i              ),
	  .DmaTxReady_i               (RdDmaTxReady_i      ),
      .DmaTxReady_1_o             (wrdt_dmatxready     ),
      .DmaTxReady_2_o             (rddsc_dmatxready    ),
	  .DmaTxReady_3_o             (rddt_dmatxready     ),
	  .DmaTxValid_1_i             (wrdt_dmatxvalid     ),
	  .DmaTxValid_2_i             (rddsc_dmatxvalid    ),  
	  .DmaTxValid_3_i             (rddt_dmatxvalid     ), 
	  .DmaTxValid_o               (RdDmaTxValid_o      ),
	  .DmaTxData_1_i              (wrdt_dmatxdata      ),
	  .DmaTxData_2_i              (rddsc_dmatxdata     ),
	  .DmaTxData_3_i              (rddt_dmatxdata      ),
	  .DmaTxData_o                (RdDmaTxData_o       )

 );


endmodule


