// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on

module data_mover_control #(
      parameter                                    DESC_ID = 8'h80,
	  parameter                                    DEVICE_FAMILY = "Arria 10"
  ) (
      input logic                                  Clk_i,
      input logic                                  Rstn_i,

      input logic   [81:0]                         MsiInterface_i,

 //////////      // AVMM Register Slave Port (Write only)   /////////////////////
      input  logic                                 DCSChipSelect_i,
      input  logic                                 DCSWrite_i,
      input  logic                                 DCSRead_i,
      input  logic  [7:0]                          DCSAddress_i,
      input  logic  [31:0]                         DCSWriteData_i,
      output logic  [31:0]                         DCSReadData_o,
      input  logic  [3:0]                          DCSByteEnable_i,
      output logic                                 DCSWaitRequest_o,
            // AVMM Register Master Port (Write only)
      output   logic [63:0]                        DCMAddress_o,
      output                                       DCMWrite_o,
      output   logic [31:0]                        DCMWriteData_o,
      output                                       DCMRead_o,
      output   logic [3:0]                         DCMByteEnable_o,
      input    logic                               DCMWaitRequest_i,
      input    logic [31:0]                        DCMReadData_i,
      input    logic                               DCMReadDataValid_i,
      /// DT 256-bit slave interface (Write only)
      input  logic                                 DTSChipSelect_i,
      input  logic                                 DTSWrite_i,
      input  logic  [4:0]                          DTSBurstCount_i,
      input  logic  [7:0]                          DTSAddress_i,
      input  logic  [255:0]                        DTSWriteData_i,
      output logic                                 DTSWaitRequest_o,
      /// DT programming interface
      output   logic  [159:0]                      DT_DmaTxData_o,
      output   logic                               DT_DmaTxValid_o,
      input    logic                               DT_DmaTxReady_i,
      /// Descriptor programming interface
      output   logic  [159:0]                      Desc_DmaTxData_o,
      output   logic                               Desc_DmaTxValid_o,
      input    logic                               Desc_DmaTxReady_i,
	  // DMA Status Interface
      input   logic  [31:0]                        DmaRxData_i,
      input   logic                                DmaRxValid_i
);

logic                                  rollover_operation_done ;      
logic                                  rollover_operation      ;      
logic [63:0]                           rc_dma_status_addr      ;      
logic [63:0]                           rc_dma_desc_addr        ;      
logic [63:0]                           ep_dma_desc_addr        ;      
logic [7:0]                            dma_lptr                ;      
logic [6:0]                            dma_lptr_plusone        ;      
logic [6:0]                            dma_table_size          ;      
logic                                  dma_control             ;      
logic                                  fetch_dt                ;      
logic                                  fetch_dt_ack            ; 
logic                                  alldesc_sent;
logic                                  status_valid;     


desc_cntl_reg status_control_reg (
    .Clk_i                         (Clk_i                  ),
    .Rstn_i                        (Rstn_i                 ),
    .DCSChipSelect_i               (DCSChipSelect_i        ),
    .DCSWrite_i                    (DCSWrite_i             ),
    .DCSRead_i                     (DCSRead_i              ),
    .DCSAddress_i                  (DCSAddress_i           ),
    .DCSWriteData_i                (DCSWriteData_i         ),
    .DCSReadData_o                 (DCSReadData_o          ),
    .DCSByteEnable_i               (DCSByteEnable_i        ),
    .DCSWaitRequest_o              (DCSWaitRequest_o       ),
	.rollover_operation_done_i     (rollover_operation_done),
	.rollover_operation_o          (rollover_operation     ),
	.rc_dma_status_addr_o          (rc_dma_status_addr     ),
	.rc_dma_desc_addr_o            (rc_dma_desc_addr       ),
	.ep_dma_desc_addr_o            (ep_dma_desc_addr       ),
	.dma_lptr_o                    (dma_lptr               ),
	.dma_lptr_plusone_o            (dma_lptr_plusone       ),
	.dma_table_size_o              (dma_table_size         ),
	.dma_control_o                 (dma_control            ),
	.fetch_dt_o                    (fetch_dt               ),
	.fetch_dt_ack_i                (fetch_dt_ack           )

 );

desc_table_fetch  # 
 (
    .DESC_ID                       (DESC_ID)
  )
 table_fetch(
    .Clk_i                    (Clk_i             ),     
    .Rstn_i                   (Rstn_i            ),
    .DmaTxData_o              (DT_DmaTxData_o    ),
    .DmaTxValid_o             (DT_DmaTxValid_o   ),
    .DmaTxReady_i             (DT_DmaTxReady_i   ),
	.rc_dma_desc_addr_i       (rc_dma_desc_addr  ),
	.ep_dma_desc_addr_i       (ep_dma_desc_addr  ),
	.dma_lptr_i               (dma_lptr          ),
	.dma_lptr_plusone_i       (dma_lptr_plusone  ),
	.dma_table_size_i         (dma_table_size    ),
	.rollover_operation_i     (rollover_operation),
	.fetch_dt_i               (fetch_dt          ),
    .fetch_dt_ack_o           (fetch_dt_ack      )
 );

desc_rcv_send #(
    .DEVICE_FAMILY             (DEVICE_FAMILY)
)
descriptor (
    .Clk_i                       (Clk_i            ),
    .Rstn_i                      (Rstn_i           ),
    .DmaTxData_o                 (Desc_DmaTxData_o ),
    .DmaTxValid_o                (desc_dmatxvalid  ),
    .DmaTxReady_i                (Desc_DmaTxReady_i),
	.Alldescriptor_sentto_mover  (alldesc_sent     ),
    .DTSChipSelect_i             (DTSChipSelect_i  ),
    .DTSWrite_i                  (DTSWrite_i       ),
    .DTSBurstCount_i             (DTSBurstCount_i  ),
    .DTSAddress_i                (DTSAddress_i     ),
    .DTSWriteData_i              (DTSWriteData_i   ),
    .DTSWaitRequest_o            (DTSWaitRequest_o )

 ); 

assign Desc_DmaTxValid_o =  desc_dmatxvalid;
//Drop Status of any id which is above 127
//ID 80 and 81 is used for descriptor table fetch, so drop these IDS. 
assign status_valid  =  ~DmaRxData_i[7] & DmaRxValid_i;

status_update #(
    .DEVICE_FAMILY             (DEVICE_FAMILY)   
)
descriptor_status (
    .Clk_i                       (Clk_i                    ),
    .Rstn_i                      (Rstn_i                   ),
    .DmaRxData_i                 (DmaRxData_i              ),
    .DmaRxValid_i                (status_valid             ),
    .DmaTxValid_i                (desc_dmatxvalid          ), 
	.DCMAddress_o                (DCMAddress_o             ),
    .DCMWrite_o                  (DCMWrite_o               ),
    .DCMWriteData_o              (DCMWriteData_o           ),
    .DCMRead_o                   (DCMRead_o                ),
    .DCMByteEnable_o             (DCMByteEnable_o          ),
    .DCMWaitRequest_i            (DCMWaitRequest_i         ),
    .DCMReadData_i               (DCMReadData_i            ),
    .DCMReadDataValid_i          (DCMReadDataValid_i       ),
    .MsiInterface_i	             (MsiInterface_i	       ),
	.Alldescriptor_sentto_mover  (alldesc_sent             ),
	.rollover_operation_done_o   (rollover_operation_done  ),
	.rollover_operation_i        (rollover_operation       ),
	.dma_control_i               (dma_control              ),
	.rc_dma_status_addr_i        (rc_dma_status_addr       ),
	.dma_lptr_i	                 (dma_lptr	               )
   
 ); 
 
 
endmodule


