// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on

module desc_cntl_reg  (
      input logic                                   Clk_i,
      input logic                                   Rstn_i,

      // DMA AVMM Register Slave Port (Write only)
      input  logic                                  DCSChipSelect_i,
      input  logic                                  DCSWrite_i,
      input  logic                                  DCSRead_i,
      input  logic  [7:0]                           DCSAddress_i,
      input  logic  [31:0]                          DCSWriteData_i,
      output  logic [31:0]                          DCSReadData_o,
      input  logic  [3:0]                           DCSByteEnable_i,
      output logic                                  DCSWaitRequest_o,

	  input  logic                                  rollover_operation_done_i,
	  output logic                                  rollover_operation_o,                        
      output logic [63:0]                           rc_dma_status_addr_o,
      output logic [63:0]                           rc_dma_desc_addr_o,
      output logic [63:0]                           ep_dma_desc_addr_o,
      output logic [7:0]                            dma_lptr_o,
      output logic [6:0]                            dma_lptr_plusone_o,
      output logic [6:0]                            dma_table_size_o,
      output logic                                  dma_control_o,
      output logic                                  fetch_dt_o,
      input  logic                                  fetch_dt_ack_i

 );

logic [31:0] rc_dma_addr_low,rc_dma_addr_high;
logic [31:0] ep_dma_addr_low,ep_dma_addr_high;
logic [31:0] dma_register;
logic [7:0]  dma_lptr;
logic [6:0]  dma_lptr_plusone;
logic [6:0]  dma_table_size;

logic  rc_dma_addr_low_cs,rc_dma_addr_high_cs,ep_dma_addr_low_cs,ep_dma_addr_high_cs;
logic  dma_lptr_cs,dma_tsize_cs,dma_cntrl_cs;

logic  rc_dma_addr_low_we,rc_dma_addr_high_we,ep_dma_addr_low_we,ep_dma_addr_high_we;
logic  dma_lptr_we,dma_tsize_we,dma_cntrl_we;

logic  dma_wait_request,dma_control,fetch_dt;
logic  rollover_write,rollover_operation_reg;
logic  dma_lptr_we_reg,rollover_cmp_reg;

// De-assert wait_request when CS is high
// Block any request till rollover operation is completed
//B;ock any request till fetch operation is not acknowledged
// Register and sent it back to Interconnect

always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    dma_wait_request <= 1'b1;
  else
    dma_wait_request <= fetch_dt | ~(dma_wait_request & DCSChipSelect_i & (DCSRead_i | DCSWrite_i)) | rollover_operation_reg;
end
assign DCSWaitRequest_o = dma_wait_request;

//DMA Register Access Decoding Logic
//No need to consider 2 LSB bits as address is DW aligned
//Only checking 3 bits as maximum register accessed are 7
assign rc_dma_addr_low_cs     = DCSChipSelect_i & (DCSAddress_i[4:2] == 3'b000) & ~fetch_dt & ~rollover_operation_reg;
assign rc_dma_addr_high_cs    = DCSChipSelect_i & (DCSAddress_i[4:2] == 3'b001) & ~fetch_dt & ~rollover_operation_reg;
assign ep_dma_addr_low_cs     = DCSChipSelect_i & (DCSAddress_i[4:2] == 3'b010) & ~fetch_dt & ~rollover_operation_reg;
assign ep_dma_addr_high_cs    = DCSChipSelect_i & (DCSAddress_i[4:2] == 3'b011) & ~fetch_dt & ~rollover_operation_reg;
assign dma_lptr_cs            = DCSChipSelect_i & (DCSAddress_i[4:2] == 3'b100) & ~fetch_dt & ~rollover_operation_reg;
assign dma_tsize_cs           = DCSChipSelect_i & (DCSAddress_i[4:2] == 3'b101) & ~fetch_dt & ~rollover_operation_reg;
assign dma_cntrl_cs           = DCSChipSelect_i & (DCSAddress_i[4:2] == 3'b110) & ~fetch_dt & ~rollover_operation_reg;
//Read DMA Register Write Access Decoding Logic
assign rc_dma_addr_low_we     = rc_dma_addr_low_cs  & DCSWrite_i;
assign rc_dma_addr_high_we    = rc_dma_addr_high_cs & DCSWrite_i;
assign ep_dma_addr_low_we     = ep_dma_addr_low_cs  & DCSWrite_i;
assign ep_dma_addr_high_we    = ep_dma_addr_high_cs & DCSWrite_i;
assign dma_lptr_we            = dma_lptr_cs         & DCSWrite_i;
assign dma_tsize_we           = dma_tsize_cs        & DCSWrite_i;
assign dma_cntrl_we           = dma_cntrl_cs        & DCSWrite_i;

//DMA  Registers
//No reset required, use D flop with CE
always_ff @(posedge Clk_i) begin
  if (rc_dma_addr_low_we)
    rc_dma_addr_low     <= DCSWriteData_i;
  if (rc_dma_addr_high_we)
    rc_dma_addr_high    <= DCSWriteData_i;
  if (ep_dma_addr_low_we)
    ep_dma_addr_low     <= DCSWriteData_i;
  if (ep_dma_addr_high_we)
    ep_dma_addr_high    <= DCSWriteData_i;
end

//DMA  Registers Read Mux
always_comb
  begin
    case(DCSAddress_i[4:2])
      3'b000 :
        dma_register      = rc_dma_addr_low;
      3'b001 :
        dma_register      = rc_dma_addr_high;
      3'b010 :
        dma_register      = ep_dma_addr_low;
      3'b011 :
        dma_register      = ep_dma_addr_high;
      3'b100 :
         dma_register     = {{24{1'b0}},dma_lptr};
      3'b101 :
        dma_register      = {{25{1'b0}},dma_table_size};
      3'b110 :
        dma_register      = {{31{1'b0}},dma_control};
      default:
        dma_register      = {32{1'b0}};

  endcase
end

always_ff @(posedge Clk_i) begin
  DCSReadData_o          <= dma_register;
end

always_ff @(posedge Clk_i) begin 
  if (~Rstn_i)
    dma_control <= 1'b0;
  else if (dma_cntrl_we)
    dma_control <= DCSWriteData_i[0];
end

always_ff @(posedge Clk_i) begin 
  if (~Rstn_i)
    dma_table_size <= 7'h7f;
  else if (dma_tsize_we)
    dma_table_size <= DCSWriteData_i[6:0];
end

always_ff @(posedge Clk_i) begin 
  if (~Rstn_i)
    dma_lptr     <= 8'hff;
  else if (dma_lptr_we)
    dma_lptr     <= DCSWriteData_i[7:0] ;      
end
//Rollover write detection
//Rollover happens when user writes last pointer equal to table size
//assign rollover_write = dma_lptr_we & ((DCSWriteData_i[6:0]) == dma_table_size);
always_ff @(posedge Clk_i) begin 
  dma_lptr_we_reg     <= dma_lptr_we  ;      
end
always_ff @(posedge Clk_i) begin 
  rollover_cmp_reg     <= ((DCSWriteData_i[6:0]) == dma_table_size)  ;      
end
assign rollover_write = dma_lptr_we_reg & rollover_cmp_reg;
//Set rollover operation in progress with rollover write and deassert with done signal
always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    rollover_operation_reg <= 1'b0;
  else 
   rollover_operation_reg <= (~rollover_operation_reg & rollover_write) | (rollover_operation_reg & ~rollover_operation_done_i);
end
//Requires to disable write before rollover operation completes
//always_ff @(posedge Clk_i) begin
//  rollover_operation_reg1 <= rollover_operation_reg;
//end
//Number of descriptors are last pointer + 1
always_ff @(posedge Clk_i) begin
  if (dma_lptr_we)
    dma_lptr_plusone     <= DCSWriteData_i[6:0] + 7'h01;
end
//Indicate Other blocks that all required registers are programmed to fetch descriptor
//Assert high with write to last pointer register
//De-assert with fetch instruction received by next module
//Keep track if more writes happens before receiving ACK from next module

always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    fetch_dt <= 1'b0;
  else
    fetch_dt <= (dma_lptr_we & ~dma_wait_request)  | (~fetch_dt_ack_i & fetch_dt);
end


adder_64bit rc_addr
(
      .Clk_i    (Clk_i),
          .a_in     ({rc_dma_addr_high,rc_dma_addr_low}),
          .b_in     (64'h000000000200),
          .sum_out  (rc_dma_desc_addr_o)

 );
//always_ff @(posedge Clk_i) begin
// rc_dma_desc_addr_o      <= {rc_dma_addr_high,rc_dma_addr_low} + 64'h000000000200;
//end
assign fetch_dt_o                     = fetch_dt;
assign rollover_operation_o           = rollover_operation_reg;
assign rc_dma_status_addr_o           = {rc_dma_addr_high,rc_dma_addr_low};
assign ep_dma_desc_addr_o             = {ep_dma_addr_high,ep_dma_addr_low};
assign dma_lptr_o                     = dma_lptr;
assign dma_lptr_plusone_o             = dma_lptr_plusone;
assign dma_table_size_o               = dma_table_size;
assign dma_control_o                  = dma_control;

endmodule
