// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on

module desc_rcv_send # (
 parameter DEVICE_FAMILY = "Arria 10"
 )
 (
      input logic                                   Clk_i,
      input logic                                   Rstn_i,

      /// DMA programming interface
      output   logic  [159:0]                       DmaTxData_o,
      output   logic                                DmaTxValid_o,
      input    logic                                DmaTxReady_i,
	  
	  output   logic                                Alldescriptor_sentto_mover,

      /// DT 256-bit slave interface (Write only)
      input  logic                                  DTSChipSelect_i,
      input  logic                                  DTSWrite_i,
      input  logic  [4:0]                           DTSBurstCount_i,
      input  logic  [7:0]                           DTSAddress_i,
      input  logic  [255:0]                         DTSWriteData_i,
      output logic                                  DTSWaitRequest_o

 );

logic         desc_fifo_wr,desc_fifo_dvalid;
logic [159:0] desc_fifo_din,desc_fifo_dout,desc_fifo_dout_reg;
logic         desc_fifo_empty,desc_fifo_rd_cmb,read_state,read_pipe1,read_pipe2,tx_valid,tx_valid_cmb;

//Descriptor Fifo Write interface
//No back pressure required/allowed on Write into Descriptor Fifo
//Software must ensure it writes when all descriptors are emptied out
assign DTSWaitRequest_o  = 1'b0;
assign desc_fifo_wr      = DTSChipSelect_i & DTSWrite_i;
assign desc_fifo_din     = DTSWriteData_i[159:0];
//Instantiate fifo module here
//Start
sync_fifo #(
      .RAMSTYLE          ("M20K"),        //Type of RAM MLAB M20K
      .AWIDTH            (8     ),       //RAM Address Width + 1
      .DEPTH             (128   ),       //Fifo Depth
      .DWIDTH            (160   ),       //RAM data Width
      .HOLDDOUT          (0     ),       //DOUT will have last read value : Only applicable when NUM_PIPE > 1
      .NUM_PIPES         (2     )        //Number of o/p registers      
) 
descriptor_fifo (
     .Clk_i               (Clk_i            ), 
     .Rstn_i              (Rstn_i           ),
     .Wren_i              (desc_fifo_wr     ),
     .Rden_i              (desc_fifo_rd_cmb ),
     .Data_i              (desc_fifo_din    ),
                                           
     .Data_o              (desc_fifo_dout   ),
     .Data_outnoreg_o     (                 ),
     .Data_valid_o        (desc_fifo_dvalid ),
     .Full_o              (                 ),
     .Empty_o             (desc_fifo_empty  ),
     .Occupancy_count_o   (                 ),
     .Available_count_o   (                 )
 );

//End

//Descriptor Fifo read interface
//Read when in read state and empty is low
assign desc_fifo_rd_cmb  = read_state & (~desc_fifo_empty);

always_ff @(posedge Clk_i) begin
  if (~Rstn_i) 
    read_state <= 1'b1;
  else
    read_state <=  (read_state & desc_fifo_empty ) | (read_pipe2 & DmaTxReady_i) ; 
end
//Read Data Pipe 1 state
always_ff @(posedge Clk_i) begin
  if (~Rstn_i) 
    read_pipe1 <= 1'b0;
  else
    read_pipe1 <= read_state & (~desc_fifo_empty) ;
end
//Read Data Pipe 2 state
always_ff @(posedge Clk_i) begin
  if (~Rstn_i) 
    read_pipe2 <= 1'b0;
  else
    read_pipe2 <= (read_pipe1 & ~read_pipe2) | (read_pipe2 & ~DmaTxReady_i)  ;
end
//Assert Tx valid when not in idle state and Tx ready high

assign tx_valid_cmb  = read_pipe2 & DmaTxReady_i;

always_ff @(posedge Clk_i) begin
  tx_valid <= tx_valid_cmb;    	  
end 
always_ff @(posedge Clk_i) begin
  if (desc_fifo_dvalid)
    desc_fifo_dout_reg <= desc_fifo_dout;    	  
end 

assign DmaTxData_o                = desc_fifo_dout_reg;
assign DmaTxValid_o               = tx_valid;
assign Alldescriptor_sentto_mover = desc_fifo_empty;

endmodule
