// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on

module rr_arbiter 

 (
      input   logic                                 Clk_i,
      input   logic                                 Rstn_i,
      
	  input   logic                                 DmaTxReady_i,
      output  logic	                                DmaTxReady_1_o,
      output  logic	                                DmaTxReady_2_o,
	  output  logic	                                DmaTxReady_3_o,
	  
	  input   logic                                 DmaTxValid_1_i,
	  input   logic                                 DmaTxValid_2_i,
	  input   logic                                 DmaTxValid_3_i,
	  output  logic                                 DmaTxValid_o,

	  input   logic [159:0]                         DmaTxData_1_i,
	  input   logic [159:0]                         DmaTxData_2_i,
	  input   logic [159:0]                         DmaTxData_3_i,
	  output  logic [159:0]                         DmaTxData_o

 );

logic  [1:0]  grant, grant_next, grant_d;
logic         DmaTxValid;
logic [159:0] DmaTxData;

assign grant_next = grant + 2'b01;

always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    grant  <= 2'b00;
  else 
    grant  <= grant_next; 
end 

always_ff @(posedge Clk_i) begin
  grant_d  <= grant; 
end 

//Give more share to read descriptor fetch
assign DmaTxReady_1_o = DmaTxReady_i & ~grant[1]  & ~grant[0];
assign DmaTxReady_2_o = DmaTxReady_i & grant[0];
assign DmaTxReady_3_o = DmaTxReady_i & grant[1] & ~grant[0];

//Block issues valid data in next clock cycles after seeing grant(Tready), so use grant_d to gate valid and data
//Give more share to read descriptor fetch
always_comb
  begin :valid_mux
    case(grant_d)
      2'b00  :
	    DmaTxValid =  DmaTxValid_1_i; 
	  2'b01  :
	    DmaTxValid =  DmaTxValid_2_i; 
	  2'b10  :
	    DmaTxValid =  DmaTxValid_3_i; 
      default:
	    DmaTxValid =  DmaTxValid_2_i; 

  endcase
end
//Give more share to read descriptor fetch
always_comb
  begin :data_mux
    case(grant_d)
      2'b00  :
	    DmaTxData =  DmaTxData_1_i; 
	  2'b01  :
	    DmaTxData =  DmaTxData_2_i; 
	  2'b10  :
	    DmaTxData =  DmaTxData_3_i; 
      default:
	    DmaTxData =  DmaTxData_2_i; 

  endcase
end

always_ff @(posedge Clk_i) begin
  DmaTxValid_o  <= DmaTxValid; 
end 

always_ff @(posedge Clk_i) begin
  DmaTxData_o  <= DmaTxData; 
end 


endmodule
