// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on

module status_update # (
 parameter DEVICE_FAMILY = "Arria 10"
 )
 (
      input   logic                                 Clk_i,
      input   logic                                 Rstn_i,

     // DMA Status Interface
      input   logic  [31:0]                         DmaRxData_i,
      input   logic                                 DmaRxValid_i,
	  
	  input   logic                                 DmaTxValid_i,

     // AVMM Status Update Master Port (Write only)
      output  logic [63:0]                         DCMAddress_o,
      output  logic                                DCMWrite_o,
      output  logic [31:0]                         DCMWriteData_o,
      output                                       DCMRead_o,
      output  logic [3:0]                          DCMByteEnable_o,
      input   logic                                DCMWaitRequest_i,
      input   logic [31:0]                         DCMReadData_i,
      input   logic                                DCMReadDataValid_i,

          //MSI Interface Information
      input   logic   [81:0]                       MsiInterface_i,
	  
      input   logic                                Alldescriptor_sentto_mover,
	  output  logic                                rollover_operation_done_o,
	  input   logic                                rollover_operation_i,  
      input   logic                                dma_control_i,
      input   logic [63:0]                         rc_dma_status_addr_i,
      input   logic [7:0]                          dma_lptr_i
 


 );

logic         status_fifo_wr,send_msi_cmb,send_msi,dma_idle,fifo_dvalid,fifo_dvalid_d1,fifo_dvalid_d2;
logic         status_fifo_dout7_d1,status_fifo_dout7_d2;
logic [7:0]   status_fifo_din,status_fifo_dout;
logic         status_fifo_empty,status_fifo_rd_cmb,idle_state;
logic         status_avmm_wr_cmb,status_avmm_wr;
logic [31:0]  msi_data,avmm_data_cmb;
logic [63:0]  msi_addr,desc_status_addr,avmm_addr_cmb;
logic         status_wr,status_last_wr; 
logic         last_wr,last_msi,last_wr_pending,all_status_recieved,statusrxcnt_cmp,rollover_operation_done;
logic  [6:0]  dma_lptr_reg,dma_lptr_hold;
logic  [8:0]  status_received_cnt,descriptor_send_cnt; // change width to 9 for linting purpose

//Status Fifo Write interface
//Hold Status Write till all descriptors are serviced
//Find out if MSI required or not and update the status in status fifo.
//If MSI is required write done status first and then write MSI on subsequent cycle in status fifo

assign last_wr        = (dma_lptr_i[6:0] == DmaRxData_i[6:0] ) & DmaRxValid_i; 
assign last_msi       = (dma_lptr_i[6:0] == DmaRxData_i[6:0] ) & MsiInterface_i[80] & DmaRxValid_i;

//Store last pointer
always_ff @(posedge Clk_i) begin
  if (last_wr)
    dma_lptr_hold <= dma_lptr_i[6:0];
end

always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    last_wr_pending <= 1'b0;
  else 
    last_wr_pending <= last_wr | (last_wr_pending & ~all_status_recieved);
end

always_ff @(posedge Clk_i) begin
  dma_lptr_reg <= dma_lptr_i[6:0];
end

always_ff @(posedge Clk_i) begin  
  if (~Rstn_i)
    status_received_cnt <= {9{1'b0}};
  else if (rollover_operation_done)
    status_received_cnt <= {9{1'b0}};
  else
    status_received_cnt <= status_received_cnt[7:0] + DmaRxValid_i;
end
always_ff @(posedge Clk_i) begin 
  if (~Rstn_i)
    descriptor_send_cnt <= {9{1'b0}};
  else if (rollover_operation_done)
    descriptor_send_cnt <= {9{1'b0}};
  else
    descriptor_send_cnt <= descriptor_send_cnt[7:0] + DmaTxValid_i;
end
always_ff @(posedge Clk_i) begin
  statusrxcnt_cmp <= (descriptor_send_cnt[7:0] == status_received_cnt[7:0]);
end
always_ff @(posedge Clk_i) begin
  all_status_recieved <= statusrxcnt_cmp & Alldescriptor_sentto_mover;
end

assign status_last_wr      = last_wr_pending & all_status_recieved;
assign send_msi_cmb        = last_wr_pending & all_status_recieved & MsiInterface_i[80];
assign status_wr           = DmaRxValid_i & dma_control_i;

always_ff @(posedge Clk_i) begin
  send_msi  <= send_msi_cmb;
end

assign status_fifo_wr      = status_wr | status_last_wr | send_msi;
assign status_fifo_din     = (status_last_wr | send_msi)? {send_msi,dma_lptr_hold} : {1'b0,DmaRxData_i[6:0]};

sync_fifo #(
      .RAMSTYLE          ("MLAB"),        //Type of RAM MLAB M20K
      .AWIDTH            (6     ),       //RAM Address Width + 1
      .DEPTH             (32    ),       //Fifo Depth
      .DWIDTH            (8     ),       //RAM data Width
      .HOLDDOUT          (0     ),       //DOUT will have last read value : Only applicable when NUM_PIPE > 1
      .NUM_PIPES         (2     )        //Number of o/p registers      
) 
status_fifo (
     .Clk_i               (Clk_i              ), 
     .Rstn_i              (Rstn_i             ), //Require Soft Reset
     .Wren_i              (status_fifo_wr     ),
     .Rden_i              (status_fifo_rd_cmb ),
     .Data_i              (status_fifo_din    ),
                                           
     .Data_o              (status_fifo_dout   ),
     .Data_outnoreg_o     (                   ),
     .Data_valid_o        (fifo_dvalid        ),
     .Full_o              (                   ),
     .Empty_o             (status_fifo_empty  ),
     .Occupancy_count_o   (                   ),
     .Available_count_o   (                   )
 );

//Descriptor Fifo read interface
//Read when in idle state and empty is low
assign status_fifo_rd_cmb  = idle_state & (~status_fifo_empty);

always_ff @(posedge Clk_i) begin 
  if (~Rstn_i)
    idle_state <= 1'b1;
  else
    idle_state <= (idle_state & status_fifo_empty) | (~DCMWaitRequest_i & status_avmm_wr);
end
//Write address and data will be stable 2 clock cycles after fifo data valid will be received
//So create avmm write based on 2 cycles delayed version of fifo data valid
always_ff @(posedge Clk_i) begin
    fifo_dvalid_d1    <= fifo_dvalid;
    fifo_dvalid_d2    <= fifo_dvalid_d1;
	
	status_fifo_dout7_d1 <= status_fifo_dout[7];
	status_fifo_dout7_d2 <= status_fifo_dout7_d1;
end
assign status_avmm_wr_cmb = fifo_dvalid_d2 | (status_avmm_wr & DCMWaitRequest_i);

always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    status_avmm_wr <= 1'b0;
  else
    status_avmm_wr  <= status_avmm_wr_cmb;
end

assign msi_data            = {{16{1'b0}},MsiInterface_i[79:64]};
assign msi_addr            = MsiInterface_i[63:0];


adder_64bit sts_addr
(
      .Clk_i    (Clk_i),
      .a_in     (rc_dma_status_addr_i),
      .b_in     ({{55{1'b0}},status_fifo_dout[6:0],2'b00}),
      .sum_out  (desc_status_addr)

 );

//assign desc_status_addr    = rc_dma_status_addr_i + {{55{1'b0}},status_fifo_dout[6:0],2'b00};

assign avmm_data_cmb       = status_fifo_dout7_d2 ? msi_data : {{31{1'b0}},1'b1};
assign avmm_addr_cmb       = status_fifo_dout7_d2 ? msi_addr : desc_status_addr;

assign DCMWrite_o          = status_avmm_wr;

always_ff @(posedge Clk_i) begin
  if (fifo_dvalid_d2)
    DCMAddress_o    <= avmm_addr_cmb;
end
always_ff @(posedge Clk_i) begin
  if (fifo_dvalid_d2)
    DCMWriteData_o  <= avmm_data_cmb;
end

assign DCMByteEnable_o = {4{1'b1}};
assign DCMRead_o       = 1'b0;

always_ff @(posedge Clk_i) begin
  rollover_operation_done <= all_status_recieved & last_wr_pending & rollover_operation_i;
end

assign rollover_operation_done_o = rollover_operation_done;

endmodule
