# This file contains pin assignments of EMIF port to suite different boards
# Currently planned board of support are S10 SoC devkit and PE Board revC

set emif_name "emif_hps"

# Pin Support Matrix for S10 SoC Devkit
# note that we have to perform a logical remapping of pins to support the HPS hard pinout.
#
# HPS Ordering    HiLo Ordering
# 2L.0  DQS4      DQS2
# 2L.1  DQS5      DQS1
# 2L.2  DQS6      DQS3
# 2L.3  DQS7      DQS0
# 2M.0  AC
# 2M.1  AC
# 2M.2  AC
# 2M.3  ECC       DQS8
# 2N.0  DQS0      DQS7
# 2N.1  DQS1      DQS5
# 2N.2  DQS2      DQS6
# 2N.3  DQS3      DQS4


set s10soc_emif_pinout_matrix [ list \
      [ list "NAME"                                "MEM"       "LOC"                  "x16_r1"   "x24_r1"   "x32_r1"   "x40_r1"   "x64_r1"   "x72_r1"   ] \
      [ list "${emif_name}_pll_ref_clk"            "both"      "PIN_M35"              "PIN_M35"  "PIN_M35"  "PIN_M35"  "PIN_M35"  "PIN_M35"  "PIN_M35"  ] \
      [ list "${emif_name}_oct_oct_rzqin"          "both"      "PIN_P34"              "PIN_P34"  "PIN_P34"  "PIN_P34"  "PIN_P34"  "PIN_P34"  "PIN_P34"  ] \
      [ list "${emif_name}_mem_mem_a[0]"           "ddr4"      "PIN_K38"              "PIN_K38"  "PIN_K38"  "PIN_K38"  "PIN_K38"  "PIN_K38"  "PIN_K38"  ] \
      [ list "${emif_name}_mem_mem_a[1]"           "ddr4"      "PIN_L37"              "PIN_L37"  "PIN_L37"  "PIN_L37"  "PIN_L37"  "PIN_L37"  "PIN_L37"  ] \
      [ list "${emif_name}_mem_mem_a[2]"           "ddr4"      "PIN_M37"              "PIN_M37"  "PIN_M37"  "PIN_M37"  "PIN_M37"  "PIN_M37"  "PIN_M37"  ] \
      [ list "${emif_name}_mem_mem_a[3]"           "ddr4"      "PIN_M38"              "PIN_M38"  "PIN_M38"  "PIN_M38"  "PIN_M38"  "PIN_M38"  "PIN_M38"  ] \
      [ list "${emif_name}_mem_mem_a[4]"           "ddr4"      "PIN_J39"              "PIN_J39"  "PIN_J39"  "PIN_J39"  "PIN_J39"  "PIN_J39"  "PIN_J39"  ] \
      [ list "${emif_name}_mem_mem_a[5]"           "ddr4"      "PIN_J38"              "PIN_J38"  "PIN_J38"  "PIN_J38"  "PIN_J38"  "PIN_J38"  "PIN_J38"  ] \
      [ list "${emif_name}_mem_mem_a[6]"           "ddr4"      "PIN_K39"              "PIN_K39"  "PIN_K39"  "PIN_K39"  "PIN_K39"  "PIN_K39"  "PIN_K39"  ] \
      [ list "${emif_name}_mem_mem_a[7]"           "ddr4"      "PIN_L39"              "PIN_L39"  "PIN_L39"  "PIN_L39"  "PIN_L39"  "PIN_L39"  "PIN_L39"  ] \
      [ list "${emif_name}_mem_mem_a[8]"           "ddr4"      "PIN_P37"              "PIN_P37"  "PIN_P37"  "PIN_P37"  "PIN_P37"  "PIN_P37"  "PIN_P37"  ] \
      [ list "${emif_name}_mem_mem_a[9]"           "ddr4"      "PIN_R37"              "PIN_R37"  "PIN_R37"  "PIN_R37"  "PIN_R37"  "PIN_R37"  "PIN_R37"  ] \
      [ list "${emif_name}_mem_mem_a[10]"          "ddr4"      "PIN_N37"              "PIN_N37"  "PIN_N37"  "PIN_N37"  "PIN_N37"  "PIN_N37"  "PIN_N37"  ] \
      [ list "${emif_name}_mem_mem_a[11]"          "ddr4"      "PIN_P38"              "PIN_P38"  "PIN_P38"  "PIN_P38"  "PIN_P38"  "PIN_P38"  "PIN_P38"  ] \
      [ list "${emif_name}_mem_mem_a[12]"          "ddr4"      "PIN_P35"              "PIN_P35"  "PIN_P35"  "PIN_P35"  "PIN_P35"  "PIN_P35"  "PIN_P35"  ] \
      [ list "${emif_name}_mem_mem_a[13]"          "ddr4"      "PIN_K36"              "PIN_K36"  "PIN_K36"  "PIN_K36"  "PIN_K36"  "PIN_K36"  "PIN_K36"  ] \
      [ list "${emif_name}_mem_mem_a[14]"          "ddr4"      "PIN_K37"              "PIN_K37"  "PIN_K37"  "PIN_K37"  "PIN_K37"  "PIN_K37"  "PIN_K37"  ] \
      [ list "${emif_name}_mem_mem_a[15]"          "ddr4"      "PIN_N36"              "PIN_N36"  "PIN_N36"  "PIN_N36"  "PIN_N36"  "PIN_N36"  "PIN_N36"  ] \
      [ list "${emif_name}_mem_mem_a[16]"          "ddr4"      "PIN_P36"              "PIN_P36"  "PIN_P36"  "PIN_P36"  "PIN_P36"  "PIN_P36"  "PIN_P36"  ] \
      [ list "${emif_name}_mem_mem_act_n[0]"       "ddr4"      "PIN_H38"              "PIN_H38"  "PIN_H38"  "PIN_H38"  "PIN_H38"  "PIN_H38"  "PIN_H38"  ] \
      [ list "${emif_name}_mem_mem_alert_n[0]"     "ddr4"      "PIN_A38"              "PIN_A38"  "PIN_A38"  "PIN_A38"  "PIN_A38"  "PIN_A38"  "PIN_A38"  ] \
      [ list "${emif_name}_mem_mem_ba[0]"          "ddr4"      "PIN_L36"              "PIN_L36"  "PIN_L36"  "PIN_L36"  "PIN_L36"  "PIN_L36"  "PIN_L36"  ] \
      [ list "${emif_name}_mem_mem_ba[1]"          "ddr4"      "PIN_T35"              "PIN_T35"  "PIN_T35"  "PIN_T35"  "PIN_T35"  "PIN_T35"  "PIN_T35"  ] \
      [ list "${emif_name}_mem_mem_bg[0]"          "ddr4"      "PIN_R36"              "PIN_R36"  "PIN_R36"  "PIN_R36"  "PIN_R36"  "PIN_R36"  "PIN_R36"  ] \
      [ list "${emif_name}_mem_mem_ck[0]"          "ddr4"      "PIN_F39"              "PIN_F39"  "PIN_F39"  "PIN_F39"  "PIN_F39"  "PIN_F39"  "PIN_F39"  ] \
      [ list "${emif_name}_mem_mem_ck_n[0]"        "ddr4"      "PIN_G39"              "PIN_G39"  "PIN_G39"  "PIN_G39"  "PIN_G39"  "PIN_G39"  "PIN_G39"  ] \
      [ list "${emif_name}_mem_mem_cke[0]"         "ddr4"      "PIN_L40"              "PIN_L40"  "PIN_L40"  "PIN_L40"  "PIN_L40"  "PIN_L40"  "PIN_L40"  ] \
      [ list "${emif_name}_mem_mem_cs_n[0]"        "ddr4"      "PIN_G38"              "PIN_G38"  "PIN_G38"  "PIN_G38"  "PIN_G38"  "PIN_G38"  "PIN_G38"  ] \
      [ list "${emif_name}_mem_mem_odt[0]"         "ddr4"      "PIN_G40"              "PIN_G40"  "PIN_G40"  "PIN_G40"  "PIN_G40"  "PIN_G40"  "PIN_G40"  ] \
      [ list "${emif_name}_mem_mem_par[0]"         "ddr4"      "PIN_H40"              "PIN_H40"  "PIN_H40"  "PIN_H40"  "PIN_H40"  "PIN_H40"  "PIN_H40"  ] \
      [ list "${emif_name}_mem_mem_reset_n[0]"     "ddr4"      "PIN_E40"              "PIN_E40"  "PIN_E40"  "PIN_E40"  "PIN_E40"  "PIN_E40"  "PIN_E40"  ] \
      [ list "${emif_name}_mem_mem_a[0]"           "ddr3"      "PIN_K38"              "PIN_K38"  "PIN_K38"  "PIN_K38"  "PIN_K38"  "PIN_K38"  "PIN_K38"  ] \
      [ list "${emif_name}_mem_mem_a[1]"           "ddr3"      "PIN_L37"              "PIN_L37"  "PIN_L37"  "PIN_L37"  "PIN_L37"  "PIN_L37"  "PIN_L37"  ] \
      [ list "${emif_name}_mem_mem_a[2]"           "ddr3"      "PIN_M37"              "PIN_M37"  "PIN_M37"  "PIN_M37"  "PIN_M37"  "PIN_M37"  "PIN_M37"  ] \
      [ list "${emif_name}_mem_mem_a[3]"           "ddr3"      "PIN_M38"              "PIN_M38"  "PIN_M38"  "PIN_M38"  "PIN_M38"  "PIN_M38"  "PIN_M38"  ] \
      [ list "${emif_name}_mem_mem_a[4]"           "ddr3"      "PIN_J39"              "PIN_J39"  "PIN_J39"  "PIN_J39"  "PIN_J39"  "PIN_J39"  "PIN_J39"  ] \
      [ list "${emif_name}_mem_mem_a[5]"           "ddr3"      "PIN_J38"              "PIN_J38"  "PIN_J38"  "PIN_J38"  "PIN_J38"  "PIN_J38"  "PIN_J38"  ] \
      [ list "${emif_name}_mem_mem_a[6]"           "ddr3"      "PIN_K39"              "PIN_K39"  "PIN_K39"  "PIN_K39"  "PIN_K39"  "PIN_K39"  "PIN_K39"  ] \
      [ list "${emif_name}_mem_mem_a[7]"           "ddr3"      "PIN_L39"              "PIN_L39"  "PIN_L39"  "PIN_L39"  "PIN_L39"  "PIN_L39"  "PIN_L39"  ] \
      [ list "${emif_name}_mem_mem_a[8]"           "ddr3"      "PIN_P37"              "PIN_P37"  "PIN_P37"  "PIN_P37"  "PIN_P37"  "PIN_P37"  "PIN_P37"  ] \
      [ list "${emif_name}_mem_mem_a[9]"           "ddr3"      "PIN_R37"              "PIN_R37"  "PIN_R37"  "PIN_R37"  "PIN_R37"  "PIN_R37"  "PIN_R37"  ] \
      [ list "${emif_name}_mem_mem_a[10]"          "ddr3"      "PIN_N37"              "PIN_N37"  "PIN_N37"  "PIN_N37"  "PIN_N37"  "PIN_N37"  "PIN_N37"  ] \
      [ list "${emif_name}_mem_mem_a[11]"          "ddr3"      "PIN_P38"              "PIN_P38"  "PIN_P38"  "PIN_P38"  "PIN_P38"  "PIN_P38"  "PIN_P38"  ] \
      [ list "${emif_name}_mem_mem_a[12]"          "ddr3"      "PIN_P35"              "PIN_P35"  "PIN_P35"  "PIN_P35"  "PIN_P35"  "PIN_P35"  "PIN_P35"  ] \
      [ list "${emif_name}_mem_mem_a[13]"          "ddr3"      "PIN_K36"              "PIN_K36"  "PIN_K36"  "PIN_K36"  "PIN_K36"  "PIN_K36"  "PIN_K36"  ] \
      [ list "${emif_name}_mem_mem_a[14]"          "ddr3"      "PIN_K37"              "PIN_K37"  "PIN_K37"  "PIN_K37"  "PIN_K37"  "PIN_K37"  "PIN_K37"  ] \
      [ list "${emif_name}_mem_mem_ba[0]"          "ddr3"      "PIN_L36"              "PIN_L36"  "PIN_L36"  "PIN_L36"  "PIN_L36"  "PIN_L36"  "PIN_L36"  ] \
      [ list "${emif_name}_mem_mem_ba[1]"          "ddr3"      "PIN_T35"              "PIN_T35"  "PIN_T35"  "PIN_T35"  "PIN_T35"  "PIN_T35"  "PIN_T35"  ] \
      [ list "${emif_name}_mem_mem_ba[2]"          "ddr3"      "PIN_R36"              "PIN_R36"  "PIN_R36"  "PIN_R36"  "PIN_R36"  "PIN_R36"  "PIN_R36"  ] \
      [ list "${emif_name}_mem_mem_cas_n"          "ddr3"      "PIN_L35"              "PIN_L35"  "PIN_L35"  "PIN_L35"  "PIN_L35"  "PIN_L35"  "PIN_L35"  ] \
      [ list "${emif_name}_mem_mem_ck[0]"          "ddr3"      "PIN_F39"              "PIN_F39"  "PIN_F39"  "PIN_F39"  "PIN_F39"  "PIN_F39"  "PIN_F39"  ] \
      [ list "${emif_name}_mem_mem_ck_n[0]"        "ddr3"      "PIN_G39"              "PIN_G39"  "PIN_G39"  "PIN_G39"  "PIN_G39"  "PIN_G39"  "PIN_G39"  ] \
      [ list "${emif_name}_mem_mem_cke[0]"         "ddr3"      "PIN_L40"              "PIN_L40"  "PIN_L40"  "PIN_L40"  "PIN_L40"  "PIN_L40"  "PIN_L40"  ] \
      [ list "${emif_name}_mem_mem_cs_n[0]"        "ddr3"      "PIN_G38"              "PIN_G38"  "PIN_G38"  "PIN_G38"  "PIN_G38"  "PIN_G38"  "PIN_G38"  ] \
      [ list "${emif_name}_mem_mem_odt[0]"         "ddr3"      "PIN_G40"              "PIN_G40"  "PIN_G40"  "PIN_G40"  "PIN_G40"  "PIN_G40"  "PIN_G40"  ] \
      [ list "${emif_name}_mem_mem_ras_n"          "ddr3"      "PIN_P36"              "PIN_P36"  "PIN_P36"  "PIN_P36"  "PIN_P36"  "PIN_P36"  "PIN_P36"  ] \
      [ list "${emif_name}_mem_mem_reset_n"        "ddr3"      "PIN_E40"              "PIN_E40"  "PIN_E40"  "PIN_E40"  "PIN_E40"  "PIN_E40"  "PIN_E40"  ] \
      [ list "${emif_name}_mem_mem_we_n[0]"        "ddr3"      "PIN_D40"              "PIN_D40"  "PIN_D40"  "PIN_D40"  "PIN_D40"  "PIN_D40"  "PIN_D40"  ] \
      [ list "${emif_name}_mem_mem_dqs[0]"         "both"      "PIN_E26"              "PIN_A36"  "PIN_A36"  "PIN_A36"  "PIN_A36"  "PIN_A36"  "PIN_A36"  ] \
      [ list "${emif_name}_mem_mem_dqs[1]"         "both"      "PIN_V28"              "PIN_E36"  "PIN_E36"  "PIN_E36"  "PIN_E36"  "PIN_E36"  "PIN_E36"  ] \
      [ list "${emif_name}_mem_mem_dqs[2]"         "both"      "PIN_T26"              unused     "PIN_R32"  "PIN_G33"  "PIN_G33"  "PIN_G33"  "PIN_G33"  ] \
      [ list "${emif_name}_mem_mem_dqs[3]"         "both"      "PIN_J26"              unused     unused     "PIN_L32"  "PIN_L32"  "PIN_L32"  "PIN_L32"  ] \
      [ list "${emif_name}_mem_mem_dqs[4]"         "both"      "PIN_L32"              unused     unused     unused     "PIN_R32"  "PIN_T26"  "PIN_T26"  ] \
      [ list "${emif_name}_mem_mem_dqs[5]"         "both"      "PIN_E36"              unused     unused     unused     unused     "PIN_V28"  "PIN_V28"  ] \
      [ list "${emif_name}_mem_mem_dqs[6]"         "both"      "PIN_G33"              unused     unused     unused     unused     "PIN_J26"  "PIN_J26"  ] \
      [ list "${emif_name}_mem_mem_dqs[7]"         "both"      "PIN_A36"              unused     unused     unused     unused     "PIN_E26"  "PIN_E26"  ] \
      [ list "${emif_name}_mem_mem_dqs[8]"         "both"      "PIN_R32"              unused     unused     unused     unused     unused     "PIN_R32"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[0]"       "both"      "PIN_F26"              "PIN_A35"  "PIN_A35"  "PIN_A35"  "PIN_A35"  "PIN_A35"  "PIN_A35"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[1]"       "both"      "PIN_V27"              "PIN_F36"  "PIN_F36"  "PIN_F36"  "PIN_F36"  "PIN_F36"  "PIN_F36"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[2]"       "both"      "PIN_R27"              unused     "PIN_T32"  "PIN_G34"  "PIN_G34"  "PIN_G34"  "PIN_G34"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[3]"       "both"      "PIN_K26"              unused     unused     "PIN_L31"  "PIN_L31"  "PIN_L31"  "PIN_L31"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[4]"       "both"      "PIN_L31"              unused     unused     unused     "PIN_T32"  "PIN_R27"  "PIN_R27"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[5]"       "both"      "PIN_F36"              unused     unused     unused     unused     "PIN_V27"  "PIN_V27"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[6]"       "both"      "PIN_G34"              unused     unused     unused     unused     "PIN_K26"  "PIN_K26"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[7]"       "both"      "PIN_A35"              unused     unused     unused     unused     "PIN_F26"  "PIN_F26"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[8]"       "both"      "PIN_T32"              unused     unused     unused     unused     unused     "PIN_T32"  ] \
      [ list "${emif_name}_mem_mem_dm[0]"          "ddr3"      "PIN_E27"              "PIN_C36"  "PIN_C36"  "PIN_C36"  "PIN_C36"  "PIN_C36"  "PIN_C36"  ] \
      [ list "${emif_name}_mem_mem_dm[1]"          "ddr3"      "PIN_V30"              "PIN_D39"  "PIN_D39"  "PIN_D39"  "PIN_D39"  "PIN_D39"  "PIN_D39"  ] \
      [ list "${emif_name}_mem_mem_dm[2]"          "ddr3"      "PIN_N25"              unused     "PIN_U34"  "PIN_F34"  "PIN_F34"  "PIN_F34"  "PIN_F34"  ] \
      [ list "${emif_name}_mem_mem_dm[3]"          "ddr3"      "PIN_L26"              unused     unused     "PIN_J34"  "PIN_J34"  "PIN_J34"  "PIN_J34"  ] \
      [ list "${emif_name}_mem_mem_dm[4]"          "ddr3"      "PIN_J34"              unused     unused     unused     "PIN_U34"  "PIN_N25"  "PIN_N25"  ] \
      [ list "${emif_name}_mem_mem_dm[5]"          "ddr3"      "PIN_D39"              unused     unused     unused     unused     "PIN_V30"  "PIN_V30"  ] \
      [ list "${emif_name}_mem_mem_dm[6]"          "ddr3"      "PIN_F34"              unused     unused     unused     unused     "PIN_L26"  "PIN_L26"  ] \
      [ list "${emif_name}_mem_mem_dm[7]"          "ddr3"      "PIN_C36"              unused     unused     unused     unused     "PIN_E27"  "PIN_E27"  ] \
      [ list "${emif_name}_mem_mem_dm[8]"          "ddr3"      "PIN_U34"              unused     unused     unused     unused     unused     "PIN_U34"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[0]"       "ddr4"      "PIN_E27"              "PIN_C36"  "PIN_C36"  "PIN_C36"  "PIN_C36"  "PIN_C36"  "PIN_C36"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[1]"       "ddr4"      "PIN_V30"              "PIN_D39"  "PIN_D39"  "PIN_D39"  "PIN_D39"  "PIN_D39"  "PIN_D39"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[2]"       "ddr4"      "PIN_N25"              unused     "PIN_U34"  "PIN_F34"  "PIN_F34"  "PIN_F34"  "PIN_F34"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[3]"       "ddr4"      "PIN_L26"              unused     unused     "PIN_J34"  "PIN_J34"  "PIN_J34"  "PIN_J34"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[4]"       "ddr4"      "PIN_J34"              unused     unused     unused     "PIN_U34"  "PIN_N25"  "PIN_N25"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[5]"       "ddr4"      "PIN_D39"              unused     unused     unused     unused     "PIN_V30"  "PIN_V30"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[6]"       "ddr4"      "PIN_F34"              unused     unused     unused     unused     "PIN_L26"  "PIN_L26"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[7]"       "ddr4"      "PIN_C36"              unused     unused     unused     unused     "PIN_E27"  "PIN_E27"  ] \
      [ list "${emif_name}_mem_mem_dbi_n[8]"       "ddr4"      "PIN_U34"              unused     unused     unused     unused     unused     "PIN_U34"  ] \
      [ list "${emif_name}_mem_mem_dq[0]"          "both"      "PIN_G27"              "PIN_A37"  "PIN_A37"  "PIN_A37"  "PIN_A37"  "PIN_A37"  "PIN_A37"  ] \
      [ list "${emif_name}_mem_mem_dq[1]"          "both"      "PIN_C27"              "PIN_B35"  "PIN_B35"  "PIN_B35"  "PIN_B35"  "PIN_B35"  "PIN_B35"  ] \
      [ list "${emif_name}_mem_mem_dq[2]"          "both"      "PIN_B27"              "PIN_D36"  "PIN_D36"  "PIN_D36"  "PIN_D36"  "PIN_D36"  "PIN_D36"  ] \
      [ list "${emif_name}_mem_mem_dq[3]"          "both"      "PIN_F27"              "PIN_B37"  "PIN_B37"  "PIN_B37"  "PIN_B37"  "PIN_B37"  "PIN_B37"  ] \
      [ list "${emif_name}_mem_mem_dq[4]"          "both"      "PIN_C26"              "PIN_B38"  "PIN_B38"  "PIN_B38"  "PIN_B38"  "PIN_B38"  "PIN_B38"  ] \
      [ list "${emif_name}_mem_mem_dq[5]"          "both"      "PIN_B25"              "PIN_C35"  "PIN_C35"  "PIN_C35"  "PIN_C35"  "PIN_C35"  "PIN_C35"  ] \
      [ list "${emif_name}_mem_mem_dq[6]"          "both"      "PIN_D26"              "PIN_C38"  "PIN_C38"  "PIN_C38"  "PIN_C38"  "PIN_C38"  "PIN_C38"  ] \
      [ list "${emif_name}_mem_mem_dq[7]"          "both"      "PIN_D25"              "PIN_C37"  "PIN_C37"  "PIN_C37"  "PIN_C37"  "PIN_C37"  "PIN_C37"  ] \
      [ list "${emif_name}_mem_mem_dq[8]"          "both"      "PIN_U30"              "PIN_H37"  "PIN_H37"  "PIN_H37"  "PIN_H37"  "PIN_H37"  "PIN_H37"  ] \
      [ list "${emif_name}_mem_mem_dq[9]"          "both"      "PIN_T30"              "PIN_E39"  "PIN_E39"  "PIN_E39"  "PIN_E39"  "PIN_E39"  "PIN_E39"  ] \
      [ list "${emif_name}_mem_mem_dq[10]"         "both"      "PIN_T29"              "PIN_F37"  "PIN_F37"  "PIN_F37"  "PIN_F37"  "PIN_F37"  "PIN_F37"  ] \
      [ list "${emif_name}_mem_mem_dq[11]"         "both"      "PIN_U28"              "PIN_E38"  "PIN_E38"  "PIN_E38"  "PIN_E38"  "PIN_E38"  "PIN_E38"  ] \
      [ list "${emif_name}_mem_mem_dq[12]"         "both"      "PIN_V25"              "PIN_D38"  "PIN_D38"  "PIN_D38"  "PIN_D38"  "PIN_D38"  "PIN_D38"  ] \
      [ list "${emif_name}_mem_mem_dq[13]"         "both"      "PIN_U27"              "PIN_D34"  "PIN_D34"  "PIN_D34"  "PIN_D34"  "PIN_D34"  "PIN_D34"  ] \
      [ list "${emif_name}_mem_mem_dq[14]"         "both"      "PIN_V26"              "PIN_D35"  "PIN_D35"  "PIN_D35"  "PIN_D35"  "PIN_D35"  "PIN_D35"  ] \
      [ list "${emif_name}_mem_mem_dq[15]"         "both"      "PIN_U29"              "PIN_E37"  "PIN_E37"  "PIN_E37"  "PIN_E37"  "PIN_E37"  "PIN_E37"  ] \
      [ list "${emif_name}_mem_mem_dq[16]"         "both"      "PIN_U25"              unused     "PIN_T34"  "PIN_H33"  "PIN_H33"  "PIN_H33"  "PIN_H33"  ] \
      [ list "${emif_name}_mem_mem_dq[17]"         "both"      "PIN_T25"              unused     "PIN_R31"  "PIN_E34"  "PIN_E34"  "PIN_E34"  "PIN_E34"  ] \
      [ list "${emif_name}_mem_mem_dq[18]"         "both"      "PIN_P25"              unused     "PIN_U33"  "PIN_F35"  "PIN_F35"  "PIN_F35"  "PIN_F35"  ] \
      [ list "${emif_name}_mem_mem_dq[19]"         "both"      "PIN_N27"              unused     "PIN_T31"  "PIN_J36"  "PIN_J36"  "PIN_J36"  "PIN_J36"  ] \
      [ list "${emif_name}_mem_mem_dq[20]"         "both"      "PIN_L25"              unused     "PIN_R34"  "PIN_G35"  "PIN_G35"  "PIN_G35"  "PIN_G35"  ] \
      [ list "${emif_name}_mem_mem_dq[21]"         "both"      "PIN_R26"              unused     "PIN_U32"  "PIN_J35"  "PIN_J35"  "PIN_J35"  "PIN_J35"  ] \
      [ list "${emif_name}_mem_mem_dq[22]"         "both"      "PIN_P26"              unused     "PIN_V32"  "PIN_H35"  "PIN_H35"  "PIN_H35"  "PIN_H35"  ] \
      [ list "${emif_name}_mem_mem_dq[23]"         "both"      "PIN_M25"              unused     "PIN_P33"  "PIN_H36"  "PIN_H36"  "PIN_H36"  "PIN_H36"  ] \
      [ list "${emif_name}_mem_mem_dq[24]"         "both"      "PIN_M27"              unused     unused     "PIN_K33"  "PIN_K33"  "PIN_K33"  "PIN_K33"  ] \
      [ list "${emif_name}_mem_mem_dq[25]"         "both"      "PIN_L27"              unused     unused     "PIN_K34"  "PIN_K34"  "PIN_K34"  "PIN_K34"  ] \
      [ list "${emif_name}_mem_mem_dq[26]"         "both"      "PIN_G25"              unused     unused     "PIN_M33"  "PIN_M33"  "PIN_M33"  "PIN_M33"  ] \
      [ list "${emif_name}_mem_mem_dq[27]"         "both"      "PIN_H25"              unused     unused     "PIN_N32"  "PIN_N32"  "PIN_N32"  "PIN_N32"  ] \
      [ list "${emif_name}_mem_mem_dq[28]"         "both"      "PIN_H27"              unused     unused     "PIN_K32"  "PIN_K32"  "PIN_K32"  "PIN_K32"  ] \
      [ list "${emif_name}_mem_mem_dq[29]"         "both"      "PIN_K27"              unused     unused     "PIN_N33"  "PIN_N33"  "PIN_N33"  "PIN_N33"  ] \
      [ list "${emif_name}_mem_mem_dq[30]"         "both"      "PIN_H26"              unused     unused     "PIN_N31"  "PIN_N31"  "PIN_N31"  "PIN_N31"  ] \
      [ list "${emif_name}_mem_mem_dq[31]"         "both"      "PIN_F25"              unused     unused     "PIN_M34"  "PIN_M34"  "PIN_M34"  "PIN_M34"  ] \
      [ list "${emif_name}_mem_mem_dq[32]"         "both"      "PIN_K33"              unused     unused     unused     "PIN_T34"  "PIN_U25"  "PIN_U25"  ] \
      [ list "${emif_name}_mem_mem_dq[33]"         "both"      "PIN_K34"              unused     unused     unused     "PIN_R31"  "PIN_T25"  "PIN_T25"  ] \
      [ list "${emif_name}_mem_mem_dq[34]"         "both"      "PIN_M33"              unused     unused     unused     "PIN_U33"  "PIN_P25"  "PIN_P25"  ] \
      [ list "${emif_name}_mem_mem_dq[35]"         "both"      "PIN_N32"              unused     unused     unused     "PIN_T31"  "PIN_N27"  "PIN_N27"  ] \
      [ list "${emif_name}_mem_mem_dq[36]"         "both"      "PIN_K32"              unused     unused     unused     "PIN_R34"  "PIN_L25"  "PIN_L25"  ] \
      [ list "${emif_name}_mem_mem_dq[37]"         "both"      "PIN_N33"              unused     unused     unused     "PIN_U32"  "PIN_R26"  "PIN_R26"  ] \
      [ list "${emif_name}_mem_mem_dq[38]"         "both"      "PIN_N31"              unused     unused     unused     "PIN_V32"  "PIN_P26"  "PIN_P26"  ] \
      [ list "${emif_name}_mem_mem_dq[39]"         "both"      "PIN_M34"              unused     unused     unused     "PIN_P33"  "PIN_M25"  "PIN_M25"  ] \
      [ list "${emif_name}_mem_mem_dq[40]"         "both"      "PIN_H37"              unused     unused     unused     unused     "PIN_U30"  "PIN_U30"  ] \
      [ list "${emif_name}_mem_mem_dq[41]"         "both"      "PIN_E39"              unused     unused     unused     unused     "PIN_T30"  "PIN_T30"  ] \
      [ list "${emif_name}_mem_mem_dq[42]"         "both"      "PIN_F37"              unused     unused     unused     unused     "PIN_T29"  "PIN_T29"  ] \
      [ list "${emif_name}_mem_mem_dq[43]"         "both"      "PIN_E38"              unused     unused     unused     unused     "PIN_U28"  "PIN_U28"  ] \
      [ list "${emif_name}_mem_mem_dq[44]"         "both"      "PIN_D38"              unused     unused     unused     unused     "PIN_V25"  "PIN_V25"  ] \
      [ list "${emif_name}_mem_mem_dq[45]"         "both"      "PIN_D34"              unused     unused     unused     unused     "PIN_U27"  "PIN_U27"  ] \
      [ list "${emif_name}_mem_mem_dq[46]"         "both"      "PIN_D35"              unused     unused     unused     unused     "PIN_V26"  "PIN_V26"  ] \
      [ list "${emif_name}_mem_mem_dq[47]"         "both"      "PIN_E37"              unused     unused     unused     unused     "PIN_U29"  "PIN_U29"  ] \
      [ list "${emif_name}_mem_mem_dq[48]"         "both"      "PIN_H33"              unused     unused     unused     unused     "PIN_M27"  "PIN_M27"  ] \
      [ list "${emif_name}_mem_mem_dq[49]"         "both"      "PIN_E34"              unused     unused     unused     unused     "PIN_L27"  "PIN_L27"  ] \
      [ list "${emif_name}_mem_mem_dq[50]"         "both"      "PIN_F35"              unused     unused     unused     unused     "PIN_G25"  "PIN_G25"  ] \
      [ list "${emif_name}_mem_mem_dq[51]"         "both"      "PIN_J36"              unused     unused     unused     unused     "PIN_H25"  "PIN_H25"  ] \
      [ list "${emif_name}_mem_mem_dq[52]"         "both"      "PIN_G35"              unused     unused     unused     unused     "PIN_H27"  "PIN_H27"  ] \
      [ list "${emif_name}_mem_mem_dq[53]"         "both"      "PIN_J35"              unused     unused     unused     unused     "PIN_K27"  "PIN_K27"  ] \
      [ list "${emif_name}_mem_mem_dq[54]"         "both"      "PIN_H35"              unused     unused     unused     unused     "PIN_H26"  "PIN_H26"  ] \
      [ list "${emif_name}_mem_mem_dq[55]"         "both"      "PIN_H36"              unused     unused     unused     unused     "PIN_F25"  "PIN_F25"  ] \
      [ list "${emif_name}_mem_mem_dq[56]"         "both"      "PIN_A37"              unused     unused     unused     unused     "PIN_G27"  "PIN_G27"  ] \
      [ list "${emif_name}_mem_mem_dq[57]"         "both"      "PIN_B35"              unused     unused     unused     unused     "PIN_C27"  "PIN_C27"  ] \
      [ list "${emif_name}_mem_mem_dq[58]"         "both"      "PIN_D36"              unused     unused     unused     unused     "PIN_B27"  "PIN_B27"  ] \
      [ list "${emif_name}_mem_mem_dq[59]"         "both"      "PIN_B37"              unused     unused     unused     unused     "PIN_F27"  "PIN_F27"  ] \
      [ list "${emif_name}_mem_mem_dq[60]"         "both"      "PIN_B38"              unused     unused     unused     unused     "PIN_C26"  "PIN_C26"  ] \
      [ list "${emif_name}_mem_mem_dq[61]"         "both"      "PIN_C35"              unused     unused     unused     unused     "PIN_B25"  "PIN_B25"  ] \
      [ list "${emif_name}_mem_mem_dq[62]"         "both"      "PIN_C38"              unused     unused     unused     unused     "PIN_D26"  "PIN_D26"  ] \
      [ list "${emif_name}_mem_mem_dq[63]"         "both"      "PIN_C37"              unused     unused     unused     unused     "PIN_D25"  "PIN_D25"  ] \
      [ list "${emif_name}_mem_mem_dq[64]"         "both"      "PIN_T34"              unused     unused     unused     unused     unused     "PIN_T34"  ] \
      [ list "${emif_name}_mem_mem_dq[65]"         "both"      "PIN_R31"              unused     unused     unused     unused     unused     "PIN_R31"  ] \
      [ list "${emif_name}_mem_mem_dq[66]"         "both"      "PIN_U33"              unused     unused     unused     unused     unused     "PIN_U33"  ] \
      [ list "${emif_name}_mem_mem_dq[67]"         "both"      "PIN_T31"              unused     unused     unused     unused     unused     "PIN_T31"  ] \
      [ list "${emif_name}_mem_mem_dq[68]"         "both"      "PIN_R34"              unused     unused     unused     unused     unused     "PIN_R34"  ] \
      [ list "${emif_name}_mem_mem_dq[69]"         "both"      "PIN_U32"              unused     unused     unused     unused     unused     "PIN_U32"  ] \
      [ list "${emif_name}_mem_mem_dq[70]"         "both"      "PIN_V32"              unused     unused     unused     unused     unused     "PIN_V32"  ] \
      [ list "${emif_name}_mem_mem_dq[71]"         "both"      "PIN_P33"              unused     unused     unused     unused     unused     "PIN_P33"  ] \
]

# Pin Support Matrix for PE1760C
# No logical remapping required, except for ECC
# HPS Ordering    HiLo Ordering
# 2L.0  DQS4      DQS4
# 2L.1  DQS5      DQS5
# 2L.2  DQS6      DQS6
# 2L.3  DQS7      DQS7
# 2M.0  AC
# 2M.1  AC
# 2M.2  AC
# 2M.3  ECC       DQS8
# 2N.0  DQS0      DQS0
# 2N.1  DQS1      DQS1
# 2N.2  DQS2      DQS2
# 2N.3  DQS3      DQS3

set pe1760c_emif_pinout_matrix [ list \
      [ list "NAME"                             "MEM"       "LOC"           "x16_r1"     "x24_r1"     "x32_r1"       "x40_r1"      "x64_r1"       "x72_r1"       "x16_r2"      "x24_r2"      "x32_r2"      "x40_r2"      "x64_r2"      "x72_r2"  ] \
      [ list "${emif_name}_pll_ref_clk"         "ddr4"      "PIN_D27"       "PIN_D27"    "PIN_D27"    "PIN_D27"      "PIN_D27"     "PIN_D27"      "PIN_D27"      "PIN_D27"     "PIN_D27"     "PIN_D27"     "PIN_D27"     "PIN_D27"     "PIN_D27" ] \
      [ list "${emif_name}_oct_oct_rzqin"       "ddr4"      "PIN_J30"       "PIN_J30"    "PIN_J30"    "PIN_J30"      "PIN_J30"     "PIN_J30"      "PIN_J30"      "PIN_J30"     "PIN_J30"     "PIN_J30"     "PIN_J30"     "PIN_J30"     "PIN_J30" ] \
      [ list "${emif_name}_mem_mem_a[0]"        "ddr4"      "PIN_M28"       "PIN_M28"    "PIN_M28"    "PIN_M28"      "PIN_M28"     "PIN_M28"      "PIN_M28"      "PIN_M28"     "PIN_M28"     "PIN_M28"     "PIN_M28"     "PIN_M28"     "PIN_M28" ] \
      [ list "${emif_name}_mem_mem_a[1]"        "ddr4"      "PIN_N28"       "PIN_N28"    "PIN_N28"    "PIN_N28"      "PIN_N28"     "PIN_N28"      "PIN_N28"      "PIN_N28"     "PIN_N28"     "PIN_N28"     "PIN_N28"     "PIN_N28"     "PIN_N28" ] \
      [ list "${emif_name}_mem_mem_a[2]"        "ddr4"      "PIN_R26"       "PIN_R26"    "PIN_R26"    "PIN_R26"      "PIN_R26"     "PIN_R26"      "PIN_R26"      "PIN_R26"     "PIN_R26"     "PIN_R26"     "PIN_R26"     "PIN_R26"     "PIN_R26" ] \
      [ list "${emif_name}_mem_mem_a[3]"        "ddr4"      "PIN_P26"       "PIN_P26"    "PIN_P26"    "PIN_P26"      "PIN_P26"     "PIN_P26"      "PIN_P26"      "PIN_P26"     "PIN_P26"     "PIN_P26"     "PIN_P26"     "PIN_P26"     "PIN_P26" ] \
      [ list "${emif_name}_mem_mem_a[4]"        "ddr4"      "PIN_P28"       "PIN_P28"    "PIN_P28"    "PIN_P28"      "PIN_P28"     "PIN_P28"      "PIN_P28"      "PIN_P28"     "PIN_P28"     "PIN_P28"     "PIN_P28"     "PIN_P28"     "PIN_P28" ] \
      [ list "${emif_name}_mem_mem_a[5]"        "ddr4"      "PIN_R27"       "PIN_R27"    "PIN_R27"    "PIN_R27"      "PIN_R27"     "PIN_R27"      "PIN_R27"      "PIN_R27"     "PIN_R27"     "PIN_R27"     "PIN_R27"     "PIN_R27"     "PIN_R27" ] \
      [ list "${emif_name}_mem_mem_a[6]"        "ddr4"      "PIN_K26"       "PIN_K26"    "PIN_K26"    "PIN_K26"      "PIN_K26"     "PIN_K26"      "PIN_K26"      "PIN_K26"     "PIN_K26"     "PIN_K26"     "PIN_K26"     "PIN_K26"     "PIN_K26" ] \
      [ list "${emif_name}_mem_mem_a[7]"        "ddr4"      "PIN_K27"       "PIN_K27"    "PIN_K27"    "PIN_K27"      "PIN_K27"     "PIN_K27"      "PIN_K27"      "PIN_K27"     "PIN_K27"     "PIN_K27"     "PIN_K27"     "PIN_K27"     "PIN_K27" ] \
      [ list "${emif_name}_mem_mem_a[8]"        "ddr4"      "PIN_N26"       "PIN_N26"    "PIN_N26"    "PIN_N26"      "PIN_N26"     "PIN_N26"      "PIN_N26"      "PIN_N26"     "PIN_N26"     "PIN_N26"     "PIN_N26"     "PIN_N26"     "PIN_N26" ] \
      [ list "${emif_name}_mem_mem_a[9]"        "ddr4"      "PIN_N27"       "PIN_N27"    "PIN_N27"    "PIN_N27"      "PIN_N27"     "PIN_N27"      "PIN_N27"      "PIN_N27"     "PIN_N27"     "PIN_N27"     "PIN_N27"     "PIN_N27"     "PIN_N27" ] \
      [ list "${emif_name}_mem_mem_a[10]"       "ddr4"      "PIN_L27"       "PIN_L27"    "PIN_L27"    "PIN_L27"      "PIN_L27"     "PIN_L27"      "PIN_L27"      "PIN_L27"     "PIN_L27"     "PIN_L27"     "PIN_L27"     "PIN_L27"     "PIN_L27" ] \
      [ list "${emif_name}_mem_mem_a[11]"       "ddr4"      "PIN_M27"       "PIN_M27"    "PIN_M27"    "PIN_M27"      "PIN_M27"     "PIN_M27"      "PIN_M27"      "PIN_M27"     "PIN_M27"     "PIN_M27"     "PIN_M27"     "PIN_M27"     "PIN_M27" ] \
      [ list "${emif_name}_mem_mem_a[12]"       "ddr4"      "PIN_J29"       "PIN_J29"    "PIN_J29"    "PIN_J29"      "PIN_J29"     "PIN_J29"      "PIN_J29"      "PIN_J29"     "PIN_J29"     "PIN_J29"     "PIN_J29"     "PIN_J29"     "PIN_J29" ] \
      [ list "${emif_name}_mem_mem_a[13]"       "ddr4"      "PIN_J28"       "PIN_J28"    "PIN_J28"    "PIN_J28"      "PIN_J28"     "PIN_J28"      "PIN_J28"      "PIN_J28"     "PIN_J28"     "PIN_J28"     "PIN_J28"     "PIN_J28"     "PIN_J28" ] \
      [ list "${emif_name}_mem_mem_a[14]"       "ddr4"      "PIN_K28"       "PIN_K28"    "PIN_K28"    "PIN_K28"      "PIN_K28"     "PIN_K28"      "PIN_K28"      "PIN_K28"     "PIN_K28"     "PIN_K28"     "PIN_K28"     "PIN_K28"     "PIN_K28" ] \
      [ list "${emif_name}_mem_mem_a[15]"       "ddr4"      "PIN_H31"       "PIN_H31"    "PIN_H31"    "PIN_H31"      "PIN_H31"     "PIN_H31"      "PIN_H31"      "PIN_H31"     "PIN_H31"     "PIN_H31"     "PIN_H31"     "PIN_H31"     "PIN_H31" ] \
      [ list "${emif_name}_mem_mem_a[16]"       "ddr4"      "PIN_H30"       "PIN_H30"    "PIN_H30"    "PIN_H30"      "PIN_H30"     "PIN_H30"      "PIN_H30"      "PIN_H30"     "PIN_H30"     "PIN_H30"     "PIN_H30"     "PIN_H30"     "PIN_H30" ] \
      [ list "${emif_name}_mem_mem_a[17]"       "ddr4"      "PIN_H28"       "PIN_H28"    "PIN_H28"    "PIN_H28"      "PIN_H28"     "PIN_H28"      "PIN_H28"      "PIN_H28"     "PIN_H28"     "PIN_H28"     "PIN_H28"     "PIN_H28"     "PIN_H28" ] \
      [ list "${emif_name}_mem_mem_ba[0]"       "ddr4"      "PIN_H27"       "PIN_H27"    "PIN_H27"    "PIN_H27"      "PIN_H27"     "PIN_H27"      "PIN_H27"      "PIN_H27"     "PIN_H27"     "PIN_H27"     "PIN_H27"     "PIN_H27"     "PIN_H27" ] \
      [ list "${emif_name}_mem_mem_ba[1]"       "ddr4"      "PIN_G27"       "PIN_G27"    "PIN_G27"    "PIN_G27"      "PIN_G27"     "PIN_G27"      "PIN_G27"      "PIN_G27"     "PIN_G27"     "PIN_G27"     "PIN_G27"     "PIN_G27"     "PIN_G27" ] \
      [ list "${emif_name}_mem_mem_bg[0]"       "ddr4"      "PIN_F27"       "PIN_F27"    "PIN_F27"    "PIN_F27"      "PIN_F27"     "PIN_F27"      "PIN_F27"      "PIN_F27"     "PIN_F27"     "PIN_F27"     "PIN_F27"     "PIN_F27"     "PIN_F27" ] \
      [ list "${emif_name}_mem_mem_bg[1]"       "ddr4"      "PIN_N30"       "PIN_N30"    "PIN_N30"    "PIN_N30"      "PIN_N30"     "PIN_N30"      "PIN_N30"      "PIN_N30"     "PIN_N30"     "PIN_N30"     "PIN_N30"     "PIN_N30"     "PIN_N30" ] \
      [ list "${emif_name}_mem_mem_act_n"       "ddr4"      "PIN_L29"       "PIN_L29"    "PIN_L29"    "PIN_L29"      "PIN_L29"     "PIN_L29"      "PIN_L29"      "PIN_L29"     "PIN_L29"     "PIN_L29"     "PIN_L29"     "PIN_L29"     "PIN_L29" ] \
      [ list "${emif_name}_mem_mem_alert_n"     "ddr4"      "PIN_B23"       "PIN_B23"    "PIN_B23"    "PIN_B23"      "PIN_B23"     "PIN_B23"      "PIN_B23"      "PIN_B23"     "PIN_B23"     "PIN_B23"     "PIN_B23"     "PIN_B23"     "PIN_B23" ] \
      [ list "${emif_name}_mem_mem_ck[0]"       "ddr4"      "PIN_K31"       "PIN_K31"    "PIN_K31"    "PIN_K31"      "PIN_K31"     "PIN_K31"      "PIN_K31"      "PIN_K31"     "PIN_K31"     "PIN_K31"     "PIN_K31"     "PIN_K31"     "PIN_K31" ] \
      [ list "${emif_name}_mem_mem_ck_n[0]"     "ddr4"      "PIN_L30"       "PIN_L30"    "PIN_L30"    "PIN_L30"      "PIN_L30"     "PIN_L30"      "PIN_L30"      "PIN_L30"     "PIN_L30"     "PIN_L30"     "PIN_L30"     "PIN_L30"     "PIN_L30" ] \
      [ list "${emif_name}_mem_mem_cke[0]"      "ddr4"      "PIN_M30"       "PIN_M30"    "PIN_M30"    "PIN_M30"      "PIN_M30"     "PIN_M30"      "PIN_M30"      "PIN_M30"     "PIN_M30"     "PIN_M30"     "PIN_M30"     "PIN_M30"     "PIN_M30" ] \
      [ list "${emif_name}_mem_mem_cs_n[0]"     "ddr4"      "PIN_M29"       "PIN_M29"    "PIN_M29"    "PIN_M29"      "PIN_M29"     "PIN_M29"      "PIN_M29"      "PIN_M29"     "PIN_M29"     "PIN_M29"     "PIN_M29"     "PIN_M29"     "PIN_M29" ] \
      [ list "${emif_name}_mem_mem_odt[0]"      "ddr4"      "PIN_K30"       "PIN_K30"    "PIN_K30"    "PIN_K30"      "PIN_K30"     "PIN_K30"      "PIN_K30"      "PIN_K30"     "PIN_K30"     "PIN_K30"     "PIN_K30"     "PIN_K30"     "PIN_K30" ] \
      [ list "${emif_name}_mem_mem_par"         "ddr4"      "PIN_P29"       "PIN_P29"    "PIN_P29"    "PIN_P29"      "PIN_P29"     "PIN_P29"      "PIN_P29"      "PIN_P29"     "PIN_P29"     "PIN_P29"     "PIN_P29"     "PIN_P29"     "PIN_P29" ] \
      [ list "${emif_name}_mem_mem_reset_n"     "ddr4"      "PIN_P31"       "PIN_P31"    "PIN_P31"    "PIN_P31"      "PIN_P31"     "PIN_P31"      "PIN_P31"      "PIN_P31"     "PIN_P31"     "PIN_P31"     "PIN_P31"     "PIN_P31"     "PIN_P31" ] \
      [ list "${emif_name}_mem_mem_cke[1]"      "ddr4"      "PIN_M31"       unused       unused       unused         unused        unused         unused         "PIN_M31"     "PIN_M31"     "PIN_M31"     "PIN_M31"     "PIN_M31"     "PIN_M31" ] \
      [ list "${emif_name}_mem_mem_cs_n[1]"     "ddr4"      "PIN_P30"       unused       unused       unused         unused        unused         unused         "PIN_P30"     "PIN_P30"     "PIN_P30"     "PIN_P30"     "PIN_P30"     "PIN_P30" ] \
      [ list "${emif_name}_mem_mem_odt[1]"      "ddr4"      "PIN_K29"       unused       unused       unused         unused        unused         unused         "PIN_K29"     "PIN_K29"     "PIN_K29"     "PIN_K29"     "PIN_K29"     "PIN_K29" ] \
      [ list "${emif_name}_mem_mem_dqs[0]"      "ddr4"      "PIN_C23"       "PIN_C23"    "PIN_C23"    "PIN_C23"      "PIN_C23"     "PIN_C23"      "PIN_C23"      "PIN_C23"     "PIN_C23"     "PIN_C23"     "PIN_C23"     "PIN_C23"     "PIN_C23" ] \
      [ list "${emif_name}_mem_mem_dqs[1]"      "ddr4"      "PIN_N25"       "PIN_N25"    "PIN_N25"    "PIN_N25"      "PIN_N25"     "PIN_N25"      "PIN_N25"      "PIN_N25"     "PIN_N25"     "PIN_N25"     "PIN_N25"     "PIN_N25"     "PIN_N25" ] \
      [ list "${emif_name}_mem_mem_dqs[2]"      "ddr4"      "PIN_T23"       unused       "PIN_B24"    "PIN_T23"      "PIN_T23"     "PIN_T23"      "PIN_T23"      unused        "PIN_B24"     "PIN_T23"     "PIN_T23"     "PIN_T23"     "PIN_T23" ] \
      [ list "${emif_name}_mem_mem_dqs[3]"      "ddr4"      "PIN_J23"       unused       unused       "PIN_J23"      "PIN_J23"     "PIN_J23"      "PIN_J23"      unused        unused        "PIN_J23"     "PIN_J23"     "PIN_J23"     "PIN_J23" ] \
      [ list "${emif_name}_mem_mem_dqs[4]"      "ddr4"      "PIN_D8"        unused       unused       unused         "PIN_B24"     "PIN_D8"       "PIN_D8"       unused        unused        unused        "PIN_B24"     "PIN_D8"      "PIN_D8"  ] \
      [ list "${emif_name}_mem_mem_dqs[5]"      "ddr4"      "PIN_N17"       unused       unused       unused         unused        "PIN_N17"      "PIN_N17"      unused        unused        unused        unused        "PIN_N17"     "PIN_N17" ] \
      [ list "${emif_name}_mem_mem_dqs[6]"      "ddr4"      "PIN_R17"       unused       unused       unused         unused        "PIN_R17"      "PIN_R17"      unused        unused        unused        unused        "PIN_R17"     "PIN_R17" ] \
      [ list "${emif_name}_mem_mem_dqs[7]"      "ddr4"      "PIN_E14"       unused       unused       unused         unused        "PIN_E14"      "PIN_E14"      unused        unused        unused        unused        "PIN_E14"     "PIN_E14" ] \
      [ list "${emif_name}_mem_mem_dqs[8]"      "ddr4"      "PIN_B24"       unused       unused       unused         unused        unused         "PIN_B24"      unused        unused        unused        unused        unused        "PIN_B24" ] \
      [ list "${emif_name}_mem_mem_dqs_n[0]"    "ddr4"      "PIN_C22"       "PIN_C22"    "PIN_C22"    "PIN_C22"      "PIN_C22"     "PIN_C22"      "PIN_C22"      "PIN_C22"     "PIN_C22"     "PIN_C22"     "PIN_C22"     "PIN_C22"     "PIN_C22" ] \
      [ list "${emif_name}_mem_mem_dqs_n[1]"    "ddr4"      "PIN_P25"       "PIN_P25"    "PIN_P25"    "PIN_P25"      "PIN_P25"     "PIN_P25"      "PIN_P25"      "PIN_P25"     "PIN_P25"     "PIN_P25"     "PIN_P25"     "PIN_P25"     "PIN_P25" ] \
      [ list "${emif_name}_mem_mem_dqs_n[2]"    "ddr4"      "PIN_R24"       unused       "PIN_B25"    "PIN_R24"      "PIN_R24"     "PIN_R24"      "PIN_R24"      unused        "PIN_B25"     "PIN_R24"     "PIN_R24"     "PIN_R24"     "PIN_R24" ] \
      [ list "${emif_name}_mem_mem_dqs_n[3]"    "ddr4"      "PIN_H23"       unused       unused       "PIN_H23"      "PIN_H23"     "PIN_H23"      "PIN_H23"      unused        unused        "PIN_H23"     "PIN_H23"     "PIN_H23"     "PIN_H23" ] \
      [ list "${emif_name}_mem_mem_dqs_n[4]"    "ddr4"      "PIN_C8"        unused       unused       unused         "PIN_B25"     "PIN_C8"       "PIN_C8"       unused        unused        unused        "PIN_B25"     "PIN_C8"      "PIN_C8"  ] \
      [ list "${emif_name}_mem_mem_dqs_n[5]"    "ddr4"      "PIN_M17"       unused       unused       unused         unused        "PIN_M17"      "PIN_M17"      unused        unused        unused        unused        "PIN_M17"     "PIN_M17" ] \
      [ list "${emif_name}_mem_mem_dqs_n[6]"    "ddr4"      "PIN_P18"       unused       unused       unused         unused        "PIN_P18"      "PIN_P18"      unused        unused        unused        unused        "PIN_P18"     "PIN_P18" ] \
      [ list "${emif_name}_mem_mem_dqs_n[7]"    "ddr4"      "PIN_F14"       unused       unused       unused         unused        "PIN_F14"      "PIN_F14"      unused        unused        unused        unused        "PIN_F14"     "PIN_F14" ] \
      [ list "${emif_name}_mem_mem_dqs_n[8]"    "ddr4"      "PIN_B25"       unused       unused       unused         unused        unused         "PIN_B25"      unused        unused        unused        unused        unused        "PIN_B25" ] \
      [ list "${emif_name}_mem_mem_dbi_n[0]"    "ddr4"      "PIN_G23"       "PIN_G23"    "PIN_G23"    "PIN_G23"      "PIN_G23"     "PIN_G23"      "PIN_G23"      "PIN_G23"     "PIN_G23"     "PIN_G23"     "PIN_G23"     "PIN_G23"     "PIN_G23" ] \
      [ list "${emif_name}_mem_mem_dbi_n[1]"    "ddr4"      "PIN_G24"       "PIN_G24"    "PIN_G24"    "PIN_G24"      "PIN_G24"     "PIN_G24"      "PIN_G24"      "PIN_G24"     "PIN_G24"     "PIN_G24"     "PIN_G24"     "PIN_G24"     "PIN_G24" ] \
      [ list "${emif_name}_mem_mem_dbi_n[2]"    "ddr4"      "PIN_P24"       unused       "PIN_F25"    "PIN_P24"      "PIN_P24"     "PIN_P24"      "PIN_P24"      unused        "PIN_F25"     "PIN_P24"     "PIN_P24"     "PIN_P24"     "PIN_P24" ] \
      [ list "${emif_name}_mem_mem_dbi_n[3]"    "ddr4"      "PIN_H22"       unused       unused       "PIN_H22"      "PIN_H22"     "PIN_H22"      "PIN_H22"      unused        unused        "PIN_H22"     "PIN_H22"     "PIN_H22"     "PIN_H22" ] \
      [ list "${emif_name}_mem_mem_dbi_n[4]"    "ddr4"      "PIN_E12"       unused       unused       unused         "PIN_F25"     "PIN_E12"      "PIN_E12"      unused        unused        unused        "PIN_F25"     "PIN_E12"     "PIN_E12" ] \
      [ list "${emif_name}_mem_mem_dbi_n[5]"    "ddr4"      "PIN_H16"       unused       unused       unused         unused        "PIN_H16"      "PIN_H16"      unused        unused        unused        unused        "PIN_H16"     "PIN_H16" ] \
      [ list "${emif_name}_mem_mem_dbi_n[6]"    "ddr4"      "PIN_L15"       unused       unused       unused         unused        "PIN_L15"      "PIN_L15"      unused        unused        unused        unused        "PIN_L15"     "PIN_L15" ] \
      [ list "${emif_name}_mem_mem_dbi_n[7]"    "ddr4"      "PIN_F15"       unused       unused       unused         unused        "PIN_F15"      "PIN_F15"      unused        unused        unused        unused        "PIN_F15"     "PIN_F15" ] \
      [ list "${emif_name}_mem_mem_dbi_n[8]"    "ddr4"      "PIN_F25"       unused       unused       unused         unused        unused         "PIN_F25"      unused        unused        unused        unused        unused        "PIN_F25" ] \
      [ list "${emif_name}_mem_mem_dq[0]"       "ddr4"      "PIN_D24"       "PIN_D24"    "PIN_D24"    "PIN_D24"      "PIN_D24"     "PIN_D24"      "PIN_D24"      "PIN_D24"     "PIN_D24"     "PIN_D24"     "PIN_D24"     "PIN_D24"     "PIN_D24" ] \
      [ list "${emif_name}_mem_mem_dq[1]"       "ddr4"      "PIN_A22"       "PIN_A22"    "PIN_A22"    "PIN_A22"      "PIN_A22"     "PIN_A22"      "PIN_A22"      "PIN_A22"     "PIN_A22"     "PIN_A22"     "PIN_A22"     "PIN_A22"     "PIN_A22" ] \
      [ list "${emif_name}_mem_mem_dq[2]"       "ddr4"      "PIN_E23"       "PIN_E23"    "PIN_E23"    "PIN_E23"      "PIN_E23"     "PIN_E23"      "PIN_E23"      "PIN_E23"     "PIN_E23"     "PIN_E23"     "PIN_E23"     "PIN_E23"     "PIN_E23" ] \
      [ list "${emif_name}_mem_mem_dq[3]"       "ddr4"      "PIN_D23"       "PIN_D23"    "PIN_D23"    "PIN_D23"      "PIN_D23"     "PIN_D23"      "PIN_D23"      "PIN_D23"     "PIN_D23"     "PIN_D23"     "PIN_D23"     "PIN_D23"     "PIN_D23" ] \
      [ list "${emif_name}_mem_mem_dq[4]"       "ddr4"      "PIN_F24"       "PIN_F24"    "PIN_F24"    "PIN_F24"      "PIN_F24"     "PIN_F24"      "PIN_F24"      "PIN_F24"     "PIN_F24"     "PIN_F24"     "PIN_F24"     "PIN_F24"     "PIN_F24" ] \
      [ list "${emif_name}_mem_mem_dq[5]"       "ddr4"      "PIN_E24"       "PIN_E24"    "PIN_E24"    "PIN_E24"      "PIN_E24"     "PIN_E24"      "PIN_E24"      "PIN_E24"     "PIN_E24"     "PIN_E24"     "PIN_E24"     "PIN_E24"     "PIN_E24" ] \
      [ list "${emif_name}_mem_mem_dq[6]"       "ddr4"      "PIN_A21"       "PIN_A21"    "PIN_A21"    "PIN_A21"      "PIN_A21"     "PIN_A21"      "PIN_A21"      "PIN_A21"     "PIN_A21"     "PIN_A21"     "PIN_A21"     "PIN_A21"     "PIN_A21" ] \
      [ list "${emif_name}_mem_mem_dq[7]"       "ddr4"      "PIN_B22"       "PIN_B22"    "PIN_B22"    "PIN_B22"      "PIN_B22"     "PIN_B22"      "PIN_B22"      "PIN_B22"     "PIN_B22"     "PIN_B22"     "PIN_B22"     "PIN_B22"     "PIN_B22" ] \
      [ list "${emif_name}_mem_mem_dq[8]"       "ddr4"      "PIN_H26"       "PIN_H26"    "PIN_H26"    "PIN_H26"      "PIN_H26"     "PIN_H26"      "PIN_H26"      "PIN_H26"     "PIN_H26"     "PIN_H26"     "PIN_H26"     "PIN_H26"     "PIN_H26" ] \
      [ list "${emif_name}_mem_mem_dq[9]"       "ddr4"      "PIN_L24"       "PIN_L24"    "PIN_L24"    "PIN_L24"      "PIN_L24"     "PIN_L24"      "PIN_L24"      "PIN_L24"     "PIN_L24"     "PIN_L24"     "PIN_L24"     "PIN_L24"     "PIN_L24" ] \
      [ list "${emif_name}_mem_mem_dq[10]"      "ddr4"      "PIN_J24"       "PIN_J24"    "PIN_J24"    "PIN_J24"      "PIN_J24"     "PIN_J24"      "PIN_J24"      "PIN_J24"     "PIN_J24"     "PIN_J24"     "PIN_J24"     "PIN_J24"     "PIN_J24" ] \
      [ list "${emif_name}_mem_mem_dq[11]"      "ddr4"      "PIN_G25"       "PIN_G25"    "PIN_G25"    "PIN_G25"      "PIN_G25"     "PIN_G25"      "PIN_G25"      "PIN_G25"     "PIN_G25"     "PIN_G25"     "PIN_G25"     "PIN_G25"     "PIN_G25" ] \
      [ list "${emif_name}_mem_mem_dq[12]"      "ddr4"      "PIN_L25"       "PIN_L25"    "PIN_L25"    "PIN_L25"      "PIN_L25"     "PIN_L25"      "PIN_L25"      "PIN_L25"     "PIN_L25"     "PIN_L25"     "PIN_L25"     "PIN_L25"     "PIN_L25" ] \
      [ list "${emif_name}_mem_mem_dq[13]"      "ddr4"      "PIN_M25"       "PIN_M25"    "PIN_M25"    "PIN_M25"      "PIN_M25"     "PIN_M25"      "PIN_M25"      "PIN_M25"     "PIN_M25"     "PIN_M25"     "PIN_M25"     "PIN_M25"     "PIN_M25" ] \
      [ list "${emif_name}_mem_mem_dq[14]"      "ddr4"      "PIN_J25"       "PIN_J25"    "PIN_J25"    "PIN_J25"      "PIN_J25"     "PIN_J25"      "PIN_J25"      "PIN_J25"     "PIN_J25"     "PIN_J25"     "PIN_J25"     "PIN_J25"     "PIN_J25" ] \
      [ list "${emif_name}_mem_mem_dq[15]"      "ddr4"      "PIN_K24"       "PIN_K24"    "PIN_K24"    "PIN_K24"      "PIN_K24"     "PIN_K24"      "PIN_K24"      "PIN_K24"     "PIN_K24"     "PIN_K24"     "PIN_K24"     "PIN_K24"     "PIN_K24" ] \
      [ list "${emif_name}_mem_mem_dq[16]"      "ddr4"      "PIN_K23"       unused       "PIN_D26"    "PIN_K23"      "PIN_K23"     "PIN_K23"      "PIN_K23"      unused        "PIN_D26"     "PIN_K23"     "PIN_K23"     "PIN_K23"     "PIN_K23" ] \
      [ list "${emif_name}_mem_mem_dq[17]"      "ddr4"      "PIN_M23"       unused       "PIN_C26"    "PIN_M23"      "PIN_M23"     "PIN_M23"      "PIN_M23"      unused        "PIN_C26"     "PIN_M23"     "PIN_M23"     "PIN_M23"     "PIN_M23" ] \
      [ list "${emif_name}_mem_mem_dq[18]"      "ddr4"      "PIN_N22"       unused       "PIN_C25"    "PIN_N22"      "PIN_N22"     "PIN_N22"      "PIN_N22"      unused        "PIN_C25"     "PIN_N22"     "PIN_N22"     "PIN_N22"     "PIN_N22" ] \
      [ list "${emif_name}_mem_mem_dq[19]"      "ddr4"      "PIN_M22"       unused       "PIN_D25"    "PIN_M22"      "PIN_M22"     "PIN_M22"      "PIN_M22"      unused        "PIN_D25"     "PIN_M22"     "PIN_M22"     "PIN_M22"     "PIN_M22" ] \
      [ list "${emif_name}_mem_mem_dq[20]"      "ddr4"      "PIN_P23"       unused       "PIN_C27"    "PIN_P23"      "PIN_P23"     "PIN_P23"      "PIN_P23"      unused        "PIN_C27"     "PIN_P23"     "PIN_P23"     "PIN_P23"     "PIN_P23" ] \
      [ list "${emif_name}_mem_mem_dq[21]"      "ddr4"      "PIN_N23"       unused       "PIN_E26"    "PIN_N23"      "PIN_N23"     "PIN_N23"      "PIN_N23"      unused        "PIN_E26"     "PIN_N23"     "PIN_N23"     "PIN_N23"     "PIN_N23" ] \
      [ list "${emif_name}_mem_mem_dq[22]"      "ddr4"      "PIN_K22"       unused       "PIN_F26"    "PIN_K22"      "PIN_K22"     "PIN_K22"      "PIN_K22"      unused        "PIN_F26"     "PIN_K22"     "PIN_K22"     "PIN_K22"     "PIN_K22" ] \
      [ list "${emif_name}_mem_mem_dq[23]"      "ddr4"      "PIN_L22"       unused       "PIN_A24"    "PIN_L22"      "PIN_L22"     "PIN_L22"      "PIN_L22"      unused        "PIN_A24"     "PIN_L22"     "PIN_L22"     "PIN_L22"     "PIN_L22" ] \
      [ list "${emif_name}_mem_mem_dq[24]"      "ddr4"      "PIN_E22"       unused       unused       "PIN_E22"      "PIN_E22"     "PIN_E22"      "PIN_E22"      unused        unused        "PIN_E22"     "PIN_E22"     "PIN_E22"     "PIN_E22" ] \
      [ list "${emif_name}_mem_mem_dq[25]"      "ddr4"      "PIN_D21"       unused       unused       "PIN_D21"      "PIN_D21"     "PIN_D21"      "PIN_D21"      unused        unused        "PIN_D21"     "PIN_D21"     "PIN_D21"     "PIN_D21" ] \
      [ list "${emif_name}_mem_mem_dq[26]"      "ddr4"      "PIN_J21"       unused       unused       "PIN_J21"      "PIN_J21"     "PIN_J21"      "PIN_J21"      unused        unused        "PIN_J21"     "PIN_J21"     "PIN_J21"     "PIN_J21" ] \
      [ list "${emif_name}_mem_mem_dq[27]"      "ddr4"      "PIN_H21"       unused       unused       "PIN_H21"      "PIN_H21"     "PIN_H21"      "PIN_H21"      unused        unused        "PIN_H21"     "PIN_H21"     "PIN_H21"     "PIN_H21" ] \
      [ list "${emif_name}_mem_mem_dq[28]"      "ddr4"      "PIN_G22"       unused       unused       "PIN_G22"      "PIN_G22"     "PIN_G22"      "PIN_G22"      unused        unused        "PIN_G22"     "PIN_G22"     "PIN_G22"     "PIN_G22" ] \
      [ list "${emif_name}_mem_mem_dq[29]"      "ddr4"      "PIN_F22"       unused       unused       "PIN_F22"      "PIN_F22"     "PIN_F22"      "PIN_F22"      unused        unused        "PIN_F22"     "PIN_F22"     "PIN_F22"     "PIN_F22" ] \
      [ list "${emif_name}_mem_mem_dq[30]"      "ddr4"      "PIN_E21"       unused       unused       "PIN_E21"      "PIN_E21"     "PIN_E21"      "PIN_E21"      unused        unused        "PIN_E21"     "PIN_E21"     "PIN_E21"     "PIN_E21" ] \
      [ list "${emif_name}_mem_mem_dq[31]"      "ddr4"      "PIN_F21"       unused       unused       "PIN_F21"      "PIN_F21"     "PIN_F21"      "PIN_F21"      unused        unused        "PIN_F21"     "PIN_F21"     "PIN_F21"     "PIN_F21" ] \
      [ list "${emif_name}_mem_mem_dq[32]"      "ddr4"      "PIN_G12"       unused       unused         unused       "PIN_D26"     "PIN_G12"      "PIN_G12"      unused        unused        unused        "PIN_D26"     "PIN_G12"     "PIN_G12" ] \
      [ list "${emif_name}_mem_mem_dq[33]"      "ddr4"      "PIN_C10"       unused       unused         unused       "PIN_C26"     "PIN_C10"      "PIN_C10"      unused        unused        unused        "PIN_C26"     "PIN_C10"     "PIN_C10" ] \
      [ list "${emif_name}_mem_mem_dq[34]"      "ddr4"      "PIN_E11"       unused       unused         unused       "PIN_C25"     "PIN_E11"      "PIN_E11"      unused        unused        unused        "PIN_C25"     "PIN_E11"     "PIN_E11" ] \
      [ list "${emif_name}_mem_mem_dq[35]"      "ddr4"      "PIN_F11"       unused       unused         unused       "PIN_D25"     "PIN_F11"      "PIN_F11"      unused        unused        unused        "PIN_D25"     "PIN_F11"     "PIN_F11" ] \
      [ list "${emif_name}_mem_mem_dq[36]"      "ddr4"      "PIN_F12"       unused       unused         unused       "PIN_C27"     "PIN_F12"      "PIN_F12"      unused        unused        unused        "PIN_C27"     "PIN_F12"     "PIN_F12" ] \
      [ list "${emif_name}_mem_mem_dq[37]"      "ddr4"      "PIN_C11"       unused       unused         unused       "PIN_E26"     "PIN_C11"      "PIN_C11"      unused        unused        unused        "PIN_E26"     "PIN_C11"     "PIN_C11" ] \
      [ list "${emif_name}_mem_mem_dq[38]"      "ddr4"      "PIN_D11"       unused       unused         unused       "PIN_F26"     "PIN_D11"      "PIN_D11"      unused        unused        unused        "PIN_F26"     "PIN_D11"     "PIN_D11" ] \
      [ list "${emif_name}_mem_mem_dq[39]"      "ddr4"      "PIN_D10"       unused       unused         unused       "PIN_A24"     "PIN_D10"      "PIN_D10"      unused        unused        unused        "PIN_A24"     "PIN_D10"     "PIN_D10" ] \
      [ list "${emif_name}_mem_mem_dq[40]"      "ddr4"      "PIN_L17"       unused       unused         unused       unused        "PIN_L17"      "PIN_L17"      unused        unused        unused        unused        "PIN_L17"     "PIN_L17" ] \
      [ list "${emif_name}_mem_mem_dq[41]"      "ddr4"      "PIN_K17"       unused       unused         unused       unused        "PIN_K17"      "PIN_K17"      unused        unused        unused        unused        "PIN_K17"     "PIN_K17" ] \
      [ list "${emif_name}_mem_mem_dq[42]"      "ddr4"      "PIN_E16"       unused       unused         unused       unused        "PIN_E16"      "PIN_E16"      unused        unused        unused        unused        "PIN_E16"     "PIN_E16" ] \
      [ list "${emif_name}_mem_mem_dq[43]"      "ddr4"      "PIN_F17"       unused       unused         unused       unused        "PIN_F17"      "PIN_F17"      unused        unused        unused        unused        "PIN_F17"     "PIN_F17" ] \
      [ list "${emif_name}_mem_mem_dq[44]"      "ddr4"      "PIN_N18"       unused       unused         unused       unused        "PIN_N18"      "PIN_N18"      unused        unused        unused        unused        "PIN_N18"     "PIN_N18" ] \
      [ list "${emif_name}_mem_mem_dq[45]"      "ddr4"      "PIN_M18"       unused       unused         unused       unused        "PIN_M18"      "PIN_M18"      unused        unused        unused        unused        "PIN_M18"     "PIN_M18" ] \
      [ list "${emif_name}_mem_mem_dq[46]"      "ddr4"      "PIN_H17"       unused       unused         unused       unused        "PIN_H17"      "PIN_H17"      unused        unused        unused        unused        "PIN_H17"     "PIN_H17" ] \
      [ list "${emif_name}_mem_mem_dq[47]"      "ddr4"      "PIN_G17"       unused       unused         unused       unused        "PIN_G17"      "PIN_G17"      unused        unused        unused        unused        "PIN_G17"     "PIN_G17" ] \
      [ list "${emif_name}_mem_mem_dq[48]"      "ddr4"      "PIN_K16"       unused       unused         unused       unused        "PIN_K16"      "PIN_K16"      unused        unused        unused        unused        "PIN_K16"     "PIN_K16" ] \
      [ list "${emif_name}_mem_mem_dq[49]"      "ddr4"      "PIN_J16"       unused       unused         unused       unused        "PIN_J16"      "PIN_J16"      unused        unused        unused        unused        "PIN_J16"     "PIN_J16" ] \
      [ list "${emif_name}_mem_mem_dq[50]"      "ddr4"      "PIN_P16"       unused       unused         unused       unused        "PIN_P16"      "PIN_P16"      unused        unused        unused        unused        "PIN_P16"     "PIN_P16" ] \
      [ list "${emif_name}_mem_mem_dq[51]"      "ddr4"      "PIN_N15"       unused       unused         unused       unused        "PIN_N15"      "PIN_N15"      unused        unused        unused        unused        "PIN_N15"     "PIN_N15" ] \
      [ list "${emif_name}_mem_mem_dq[52]"      "ddr4"      "PIN_N16"       unused       unused         unused       unused        "PIN_N16"      "PIN_N16"      unused        unused        unused        unused        "PIN_N16"     "PIN_N16" ] \
      [ list "${emif_name}_mem_mem_dq[53]"      "ddr4"      "PIN_L16"       unused       unused         unused       unused        "PIN_L16"      "PIN_L16"      unused        unused        unused        unused        "PIN_L16"     "PIN_L16" ] \
      [ list "${emif_name}_mem_mem_dq[54]"      "ddr4"      "PIN_P15"       unused       unused         unused       unused        "PIN_P15"      "PIN_P15"      unused        unused        unused        unused        "PIN_P15"     "PIN_P15" ] \
      [ list "${emif_name}_mem_mem_dq[55]"      "ddr4"      "PIN_M15"       unused       unused         unused       unused        "PIN_M15"      "PIN_M15"      unused        unused        unused        unused        "PIN_M15"     "PIN_M15" ] \
      [ list "${emif_name}_mem_mem_dq[56]"      "ddr4"      "PIN_D15"       unused       unused         unused       unused        "PIN_D15"      "PIN_D15"      unused        unused        unused        unused        "PIN_D15"     "PIN_D15" ] \
      [ list "${emif_name}_mem_mem_dq[57]"      "ddr4"      "PIN_D14"       unused       unused         unused       unused        "PIN_D14"      "PIN_D14"      unused        unused        unused        unused        "PIN_D14"     "PIN_D14" ] \
      [ list "${emif_name}_mem_mem_dq[58]"      "ddr4"      "PIN_G13"       unused       unused         unused       unused        "PIN_G13"      "PIN_G13"      unused        unused        unused        unused        "PIN_G13"     "PIN_G13" ] \
      [ list "${emif_name}_mem_mem_dq[59]"      "ddr4"      "PIN_G14"       unused       unused         unused       unused        "PIN_G14"      "PIN_G14"      unused        unused        unused        unused        "PIN_G14"     "PIN_G14" ] \
      [ list "${emif_name}_mem_mem_dq[60]"      "ddr4"      "PIN_J15"       unused       unused         unused       unused        "PIN_J15"      "PIN_J15"      unused        unused        unused        unused        "PIN_J15"     "PIN_J15" ] \
      [ list "${emif_name}_mem_mem_dq[61]"      "ddr4"      "PIN_H15"       unused       unused         unused       unused        "PIN_H15"      "PIN_H15"      unused        unused        unused        unused        "PIN_H15"     "PIN_H15" ] \
      [ list "${emif_name}_mem_mem_dq[62]"      "ddr4"      "PIN_E13"       unused       unused         unused       unused        "PIN_E13"      "PIN_E13"      unused        unused        unused        unused        "PIN_E13"     "PIN_E13" ] \
      [ list "${emif_name}_mem_mem_dq[63]"      "ddr4"      "PIN_G15"       unused       unused         unused       unused        "PIN_G15"      "PIN_G15"      unused        unused        unused        unused        "PIN_G15"     "PIN_G15" ] \
      [ list "${emif_name}_mem_mem_dq[64]"      "ddr4"      "PIN_D26"       unused       unused         unused       unused        unused         "PIN_D26"      unused        unused        unused        unused        unused        "PIN_D26" ] \
      [ list "${emif_name}_mem_mem_dq[65]"      "ddr4"      "PIN_C26"       unused       unused         unused       unused        unused         "PIN_C26"      unused        unused        unused        unused        unused        "PIN_C26" ] \
      [ list "${emif_name}_mem_mem_dq[66]"      "ddr4"      "PIN_C25"       unused       unused         unused       unused        unused         "PIN_C25"      unused        unused        unused        unused        unused        "PIN_C25" ] \
      [ list "${emif_name}_mem_mem_dq[67]"      "ddr4"      "PIN_D25"       unused       unused         unused       unused        unused         "PIN_D25"      unused        unused        unused        unused        unused        "PIN_D25" ] \
      [ list "${emif_name}_mem_mem_dq[68]"      "ddr4"      "PIN_C27"       unused       unused         unused       unused        unused         "PIN_C27"      unused        unused        unused        unused        unused        "PIN_C27" ] \
      [ list "${emif_name}_mem_mem_dq[69]"      "ddr4"      "PIN_E26"       unused       unused         unused       unused        unused         "PIN_E26"      unused        unused        unused        unused        unused        "PIN_E26" ] \
      [ list "${emif_name}_mem_mem_dq[70]"      "ddr4"      "PIN_F26"       unused       unused         unused       unused        unused         "PIN_F26"      unused        unused        unused        unused        unused        "PIN_F26" ] \
      [ list "${emif_name}_mem_mem_dq[71]"      "ddr4"      "PIN_A24"       unused       unused         unused       unused        unused         "PIN_A24"      unused        unused        unused        unused        unused        "PIN_A24" ] \
]

set atso12_emif_pinout_matrix [ list \
      [ list "NAME"                             "MEM"       "LOC"           "x16_r1"     "x24_r1"     "x32_r1"       "x40_r1"      "x16_r2"      "x24_r2"      "x32_r2"      "x40_r2"      ] \
      [ list "${emif_name}_pll_ref_clk"         "ddr4"      "PIN_D27"       "PIN_D27"    "PIN_D27"    "PIN_D27"      "PIN_D27"     "PIN_D27"     "PIN_D27"     "PIN_D27"     "PIN_D27"     ] \
      [ list "${emif_name}_oct_oct_rzqin"       "ddr4"      "PIN_J30"       "PIN_J30"    "PIN_J30"    "PIN_J30"      "PIN_J30"     "PIN_J30"     "PIN_J30"     "PIN_J30"     "PIN_J30"     ] \
      [ list "${emif_name}_mem_mem_a[0]"        "ddr4"      "PIN_M28"       "PIN_M28"    "PIN_M28"    "PIN_M28"      "PIN_M28"     "PIN_M28"     "PIN_M28"     "PIN_M28"     "PIN_M28"     ] \
      [ list "${emif_name}_mem_mem_a[1]"        "ddr4"      "PIN_N28"       "PIN_N28"    "PIN_N28"    "PIN_N28"      "PIN_N28"     "PIN_N28"     "PIN_N28"     "PIN_N28"     "PIN_N28"     ] \
      [ list "${emif_name}_mem_mem_a[2]"        "ddr4"      "PIN_R26"       "PIN_R26"    "PIN_R26"    "PIN_R26"      "PIN_R26"     "PIN_R26"     "PIN_R26"     "PIN_R26"     "PIN_R26"     ] \
      [ list "${emif_name}_mem_mem_a[3]"        "ddr4"      "PIN_P26"       "PIN_P26"    "PIN_P26"    "PIN_P26"      "PIN_P26"     "PIN_P26"     "PIN_P26"     "PIN_P26"     "PIN_P26"     ] \
      [ list "${emif_name}_mem_mem_a[4]"        "ddr4"      "PIN_P28"       "PIN_P28"    "PIN_P28"    "PIN_P28"      "PIN_P28"     "PIN_P28"     "PIN_P28"     "PIN_P28"     "PIN_P28"     ] \
      [ list "${emif_name}_mem_mem_a[5]"        "ddr4"      "PIN_R27"       "PIN_R27"    "PIN_R27"    "PIN_R27"      "PIN_R27"     "PIN_R27"     "PIN_R27"     "PIN_R27"     "PIN_R27"     ] \
      [ list "${emif_name}_mem_mem_a[6]"        "ddr4"      "PIN_K26"       "PIN_K26"    "PIN_K26"    "PIN_K26"      "PIN_K26"     "PIN_K26"     "PIN_K26"     "PIN_K26"     "PIN_K26"     ] \
      [ list "${emif_name}_mem_mem_a[7]"        "ddr4"      "PIN_K27"       "PIN_K27"    "PIN_K27"    "PIN_K27"      "PIN_K27"     "PIN_K27"     "PIN_K27"     "PIN_K27"     "PIN_K27"     ] \
      [ list "${emif_name}_mem_mem_a[8]"        "ddr4"      "PIN_N26"       "PIN_N26"    "PIN_N26"    "PIN_N26"      "PIN_N26"     "PIN_N26"     "PIN_N26"     "PIN_N26"     "PIN_N26"     ] \
      [ list "${emif_name}_mem_mem_a[9]"        "ddr4"      "PIN_N27"       "PIN_N27"    "PIN_N27"    "PIN_N27"      "PIN_N27"     "PIN_N27"     "PIN_N27"     "PIN_N27"     "PIN_N27"     ] \
      [ list "${emif_name}_mem_mem_a[10]"       "ddr4"      "PIN_L27"       "PIN_L27"    "PIN_L27"    "PIN_L27"      "PIN_L27"     "PIN_L27"     "PIN_L27"     "PIN_L27"     "PIN_L27"     ] \
      [ list "${emif_name}_mem_mem_a[11]"       "ddr4"      "PIN_M27"       "PIN_M27"    "PIN_M27"    "PIN_M27"      "PIN_M27"     "PIN_M27"     "PIN_M27"     "PIN_M27"     "PIN_M27"     ] \
      [ list "${emif_name}_mem_mem_a[12]"       "ddr4"      "PIN_J29"       "PIN_J29"    "PIN_J29"    "PIN_J29"      "PIN_J29"     "PIN_J29"     "PIN_J29"     "PIN_J29"     "PIN_J29"     ] \
      [ list "${emif_name}_mem_mem_a[13]"       "ddr4"      "PIN_J28"       "PIN_J28"    "PIN_J28"    "PIN_J28"      "PIN_J28"     "PIN_J28"     "PIN_J28"     "PIN_J28"     "PIN_J28"     ] \
      [ list "${emif_name}_mem_mem_a[14]"       "ddr4"      "PIN_K28"       "PIN_K28"    "PIN_K28"    "PIN_K28"      "PIN_K28"     "PIN_K28"     "PIN_K28"     "PIN_K28"     "PIN_K28"     ] \
      [ list "${emif_name}_mem_mem_a[15]"       "ddr4"      "PIN_H31"       "PIN_H31"    "PIN_H31"    "PIN_H31"      "PIN_H31"     "PIN_H31"     "PIN_H31"     "PIN_H31"     "PIN_H31"     ] \
      [ list "${emif_name}_mem_mem_a[16]"       "ddr4"      "PIN_H30"       "PIN_H30"    "PIN_H30"    "PIN_H30"      "PIN_H30"     "PIN_H30"     "PIN_H30"     "PIN_H30"     "PIN_H30"     ] \
      [ list "${emif_name}_mem_mem_ba[0]"       "ddr4"      "PIN_H27"       "PIN_H27"    "PIN_H27"    "PIN_H27"      "PIN_H27"     "PIN_H27"     "PIN_H27"     "PIN_H27"     "PIN_H27"     ] \
      [ list "${emif_name}_mem_mem_ba[1]"       "ddr4"      "PIN_G27"       "PIN_G27"    "PIN_G27"    "PIN_G27"      "PIN_G27"     "PIN_G27"     "PIN_G27"     "PIN_G27"     "PIN_G27"     ] \
      [ list "${emif_name}_mem_mem_bg[0]"       "ddr4"      "PIN_F27"       "PIN_F27"    "PIN_F27"    "PIN_F27"      "PIN_F27"     "PIN_F27"     "PIN_F27"     "PIN_F27"     "PIN_F27"     ] \
      [ list "${emif_name}_mem_mem_bg[1]"       "ddr4"      "PIN_N30"       "PIN_N30"    "PIN_N30"    "PIN_N30"      "PIN_N30"     "PIN_N30"     "PIN_N30"     "PIN_N30"     "PIN_N30"     ] \
      [ list "${emif_name}_mem_mem_act_n"       "ddr4"      "PIN_L29"       "PIN_L29"    "PIN_L29"    "PIN_L29"      "PIN_L29"     "PIN_L29"     "PIN_L29"     "PIN_L29"     "PIN_L29"     ] \
      [ list "${emif_name}_mem_mem_alert_n"     "ddr4"      "PIN_B23"       "PIN_B23"    "PIN_B23"    "PIN_B23"      "PIN_B23"     "PIN_B23"     "PIN_B23"     "PIN_B23"     "PIN_B23"     ] \
      [ list "${emif_name}_mem_mem_ck[0]"       "ddr4"      "PIN_K31"       "PIN_K31"    "PIN_K31"    "PIN_K31"      "PIN_K31"     "PIN_K31"     "PIN_K31"     "PIN_K31"     "PIN_K31"     ] \
      [ list "${emif_name}_mem_mem_ck_n[0]"     "ddr4"      "PIN_L30"       "PIN_L30"    "PIN_L30"    "PIN_L30"      "PIN_L30"     "PIN_L30"     "PIN_L30"     "PIN_L30"     "PIN_L30"     ] \
      [ list "${emif_name}_mem_mem_cke[0]"      "ddr4"      "PIN_M30"       "PIN_M30"    "PIN_M30"    "PIN_M30"      "PIN_M30"     "PIN_M30"     "PIN_M30"     "PIN_M30"     "PIN_M30"     ] \
      [ list "${emif_name}_mem_mem_cs_n[0]"     "ddr4"      "PIN_M29"       "PIN_M29"    "PIN_M29"    "PIN_M29"      "PIN_M29"     "PIN_M29"     "PIN_M29"     "PIN_M29"     "PIN_M29"     ] \
      [ list "${emif_name}_mem_mem_odt[0]"      "ddr4"      "PIN_K30"       "PIN_K30"    "PIN_K30"    "PIN_K30"      "PIN_K30"     "PIN_K30"     "PIN_K30"     "PIN_K30"     "PIN_K30"     ] \
      [ list "${emif_name}_mem_mem_par"         "ddr4"      "PIN_P29"       "PIN_P29"    "PIN_P29"    "PIN_P29"      "PIN_P29"     "PIN_P29"     "PIN_P29"     "PIN_P29"     "PIN_P29"     ] \
      [ list "${emif_name}_mem_mem_reset_n"     "ddr4"      "PIN_P31"       "PIN_P31"    "PIN_P31"    "PIN_P31"      "PIN_P31"     "PIN_P31"     "PIN_P31"     "PIN_P31"     "PIN_P31"     ] \
      [ list "${emif_name}_mem_mem_cke[1]"      "ddr4"      "PIN_M31"       unused       unused       unused         unused        "PIN_M31"     "PIN_M31"     "PIN_M31"     "PIN_M31"     ] \
      [ list "${emif_name}_mem_mem_cs_n[1]"     "ddr4"      "PIN_P30"       unused       unused       unused         unused        "PIN_P30"     "PIN_P30"     "PIN_P30"     "PIN_P30"     ] \
      [ list "${emif_name}_mem_mem_odt[1]"      "ddr4"      "PIN_K29"       unused       unused       unused         unused        "PIN_K29"     "PIN_K29"     "PIN_K29"     "PIN_K29"     ] \
      [ list "${emif_name}_mem_mem_dqs[0]"      "ddr4"      "PIN_C23"       "PIN_C23"    "PIN_C23"    "PIN_C23"      "PIN_C23"     "PIN_C23"     "PIN_C23"     "PIN_C23"     "PIN_C23"     ] \
      [ list "${emif_name}_mem_mem_dqs[1]"      "ddr4"      "PIN_N25"       "PIN_N25"    "PIN_N25"    "PIN_N25"      "PIN_N25"     "PIN_N25"     "PIN_N25"     "PIN_N25"     "PIN_N25"     ] \
      [ list "${emif_name}_mem_mem_dqs[2]"      "ddr4"      "PIN_T23"       unused       "PIN_B24"    "PIN_T23"      "PIN_T23"     unused        "PIN_B24"     "PIN_T23"     "PIN_T23"     ] \
      [ list "${emif_name}_mem_mem_dqs[3]"      "ddr4"      "PIN_J23"       unused       unused       "PIN_J23"      "PIN_J23"     unused        unused        "PIN_J23"     "PIN_J23"     ] \
      [ list "${emif_name}_mem_mem_dqs[4]"      "ddr4"      "PIN_D8"        unused       unused       unused         "PIN_B24"     unused        unused        unused        "PIN_B24"     ] \
      [ list "${emif_name}_mem_mem_dqs_n[0]"    "ddr4"      "PIN_C22"       "PIN_C22"    "PIN_C22"    "PIN_C22"      "PIN_C22"     "PIN_C22"     "PIN_C22"     "PIN_C22"     "PIN_C22"     ] \
      [ list "${emif_name}_mem_mem_dqs_n[1]"    "ddr4"      "PIN_P25"       "PIN_P25"    "PIN_P25"    "PIN_P25"      "PIN_P25"     "PIN_P25"     "PIN_P25"     "PIN_P25"     "PIN_P25"     ] \
      [ list "${emif_name}_mem_mem_dqs_n[2]"    "ddr4"      "PIN_R24"       unused       "PIN_B25"    "PIN_R24"      "PIN_R24"     unused        "PIN_B25"     "PIN_R24"     "PIN_R24"     ] \
      [ list "${emif_name}_mem_mem_dqs_n[3]"    "ddr4"      "PIN_H23"       unused       unused       "PIN_H23"      "PIN_H23"     unused        unused        "PIN_H23"     "PIN_H23"     ] \
      [ list "${emif_name}_mem_mem_dqs_n[4]"    "ddr4"      "PIN_C8"        unused       unused       unused         "PIN_B25"     unused        unused        unused        "PIN_B25"     ] \
      [ list "${emif_name}_mem_mem_dbi_n[0]"    "ddr4"      "PIN_G23"       "PIN_G23"    "PIN_G23"    "PIN_G23"      "PIN_G23"     "PIN_G23"     "PIN_G23"     "PIN_G23"     "PIN_G23"     ] \
      [ list "${emif_name}_mem_mem_dbi_n[1]"    "ddr4"      "PIN_G24"       "PIN_G24"    "PIN_G24"    "PIN_G24"      "PIN_G24"     "PIN_G24"     "PIN_G24"     "PIN_G24"     "PIN_G24"     ] \
      [ list "${emif_name}_mem_mem_dbi_n[2]"    "ddr4"      "PIN_P24"       unused       "PIN_F25"    "PIN_P24"      "PIN_P24"     unused        "PIN_F25"     "PIN_P24"     "PIN_P24"     ] \
      [ list "${emif_name}_mem_mem_dbi_n[3]"    "ddr4"      "PIN_H22"       unused       unused       "PIN_H22"      "PIN_H22"     unused        unused        "PIN_H22"     "PIN_H22"     ] \
      [ list "${emif_name}_mem_mem_dbi_n[4]"    "ddr4"      "PIN_E12"       unused       unused       unused         "PIN_F25"     unused        unused        unused        "PIN_F25"     ] \
      [ list "${emif_name}_mem_mem_dq[0]"       "ddr4"      "PIN_E24"       "PIN_E24"    "PIN_E24"    "PIN_E24"      "PIN_E24"     "PIN_E24"     "PIN_E24"     "PIN_E24"     "PIN_E24"     ] \
      [ list "${emif_name}_mem_mem_dq[1]"       "ddr4"      "PIN_A22"       "PIN_A22"    "PIN_A22"    "PIN_A22"      "PIN_A22"     "PIN_A22"     "PIN_A22"     "PIN_A22"     "PIN_A22"     ] \
      [ list "${emif_name}_mem_mem_dq[2]"       "ddr4"      "PIN_E23"       "PIN_E23"    "PIN_E23"    "PIN_E23"      "PIN_E23"     "PIN_E23"     "PIN_E23"     "PIN_E23"     "PIN_E23"     ] \
      [ list "${emif_name}_mem_mem_dq[3]"       "ddr4"      "PIN_F24"       "PIN_F24"    "PIN_F24"    "PIN_F24"      "PIN_F24"     "PIN_F24"     "PIN_F24"     "PIN_F24"     "PIN_F24"     ] \
      [ list "${emif_name}_mem_mem_dq[4]"       "ddr4"      "PIN_D24"       "PIN_D24"    "PIN_D24"    "PIN_D24"      "PIN_D24"     "PIN_D24"     "PIN_D24"     "PIN_D24"     "PIN_D24"     ] \
      [ list "${emif_name}_mem_mem_dq[5]"       "ddr4"      "PIN_A21"       "PIN_A21"    "PIN_A21"    "PIN_A21"      "PIN_A21"     "PIN_A21"     "PIN_A21"     "PIN_A21"     "PIN_A21"     ] \
      [ list "${emif_name}_mem_mem_dq[6]"       "ddr4"      "PIN_B22"       "PIN_B22"    "PIN_B22"    "PIN_B22"      "PIN_B22"     "PIN_B22"     "PIN_B22"     "PIN_B22"     "PIN_B22"     ] \
      [ list "${emif_name}_mem_mem_dq[7]"       "ddr4"      "PIN_D23"       "PIN_D23"    "PIN_D23"    "PIN_D23"      "PIN_D23"     "PIN_D23"     "PIN_D23"     "PIN_D23"     "PIN_D23"     ] \
      [ list "${emif_name}_mem_mem_dq[8]"       "ddr4"      "PIN_H26"       "PIN_H26"    "PIN_H26"    "PIN_H26"      "PIN_H26"     "PIN_H26"     "PIN_H26"     "PIN_H26"     "PIN_H26"     ] \
      [ list "${emif_name}_mem_mem_dq[9]"       "ddr4"      "PIN_G25"       "PIN_G25"    "PIN_G25"    "PIN_G25"      "PIN_G25"     "PIN_G25"     "PIN_G25"     "PIN_G25"     "PIN_G25"     ] \
      [ list "${emif_name}_mem_mem_dq[10]"      "ddr4"      "PIN_J24"       "PIN_J24"    "PIN_J24"    "PIN_J24"      "PIN_J24"     "PIN_J24"     "PIN_J24"     "PIN_J24"     "PIN_J24"     ] \
      [ list "${emif_name}_mem_mem_dq[11]"      "ddr4"      "PIN_J25"       "PIN_J25"    "PIN_J25"    "PIN_J25"      "PIN_J25"     "PIN_J25"     "PIN_J25"     "PIN_J25"     "PIN_J25"     ] \
      [ list "${emif_name}_mem_mem_dq[12]"      "ddr4"      "PIN_K24"       "PIN_K24"    "PIN_K24"    "PIN_K24"      "PIN_K24"     "PIN_K24"     "PIN_K24"     "PIN_K24"     "PIN_K24"     ] \
      [ list "${emif_name}_mem_mem_dq[13]"      "ddr4"      "PIN_M25"       "PIN_M25"    "PIN_M25"    "PIN_M25"      "PIN_M25"     "PIN_M25"     "PIN_M25"     "PIN_M25"     "PIN_M25"     ] \
      [ list "${emif_name}_mem_mem_dq[14]"      "ddr4"      "PIN_L25"       "PIN_L25"    "PIN_L25"    "PIN_L25"      "PIN_L25"     "PIN_L25"     "PIN_L25"     "PIN_L25"     "PIN_L25"     ] \
      [ list "${emif_name}_mem_mem_dq[15]"      "ddr4"      "PIN_L24"       "PIN_L24"    "PIN_L24"    "PIN_L24"      "PIN_L24"     "PIN_L24"     "PIN_L24"     "PIN_L24"     "PIN_L24"     ] \
      [ list "${emif_name}_mem_mem_dq[16]"      "ddr4"      "PIN_K23"       unused       "PIN_A24"    "PIN_K23"      "PIN_K23"     unused        "PIN_A24"     "PIN_K23"     "PIN_K23"     ] \
      [ list "${emif_name}_mem_mem_dq[17]"      "ddr4"      "PIN_K22"       unused       "PIN_D25"    "PIN_K22"      "PIN_K22"     unused        "PIN_D25"     "PIN_K22"     "PIN_K22"     ] \
      [ list "${emif_name}_mem_mem_dq[18]"      "ddr4"      "PIN_M23"       unused       "PIN_C25"    "PIN_M23"      "PIN_M23"     unused        "PIN_C25"     "PIN_M23"     "PIN_M23"     ] \
      [ list "${emif_name}_mem_mem_dq[19]"      "ddr4"      "PIN_L22"       unused       "PIN_D26"    "PIN_L22"      "PIN_L22"     unused        "PIN_D26"     "PIN_L22"     "PIN_L22"     ] \
      [ list "${emif_name}_mem_mem_dq[20]"      "ddr4"      "PIN_M22"       unused       "PIN_E26"    "PIN_M22"      "PIN_M22"     unused        "PIN_E26"     "PIN_M22"     "PIN_M22"     ] \
      [ list "${emif_name}_mem_mem_dq[21]"      "ddr4"      "PIN_P23"       unused       "PIN_C27"    "PIN_P23"      "PIN_P23"     unused        "PIN_C27"     "PIN_P23"     "PIN_P23"     ] \
      [ list "${emif_name}_mem_mem_dq[22]"      "ddr4"      "PIN_N23"       unused       "PIN_C26"    "PIN_N23"      "PIN_N23"     unused        "PIN_C26"     "PIN_N23"     "PIN_N23"     ] \
      [ list "${emif_name}_mem_mem_dq[23]"      "ddr4"      "PIN_N22"       unused       "PIN_F26"    "PIN_N22"      "PIN_N22"     unused        "PIN_F26"     "PIN_N22"     "PIN_N22"     ] \
      [ list "${emif_name}_mem_mem_dq[24]"      "ddr4"      "PIN_D21"       unused       unused       "PIN_D21"      "PIN_D21"     unused        unused        "PIN_D21"     "PIN_D21"     ] \
      [ list "${emif_name}_mem_mem_dq[25]"      "ddr4"      "PIN_E21"       unused       unused       "PIN_E21"      "PIN_E21"     unused        unused        "PIN_E21"     "PIN_E21"     ] \
      [ list "${emif_name}_mem_mem_dq[26]"      "ddr4"      "PIN_F21"       unused       unused       "PIN_F21"      "PIN_F21"     unused        unused        "PIN_F21"     "PIN_F21"     ] \
      [ list "${emif_name}_mem_mem_dq[27]"      "ddr4"      "PIN_E22"       unused       unused       "PIN_E22"      "PIN_E22"     unused        unused        "PIN_E22"     "PIN_E22"     ] \
      [ list "${emif_name}_mem_mem_dq[28]"      "ddr4"      "PIN_F22"       unused       unused       "PIN_F22"      "PIN_F22"     unused        unused        "PIN_F22"     "PIN_F22"     ] \
      [ list "${emif_name}_mem_mem_dq[29]"      "ddr4"      "PIN_H21"       unused       unused       "PIN_H21"      "PIN_H21"     unused        unused        "PIN_H21"     "PIN_H21"     ] \
      [ list "${emif_name}_mem_mem_dq[30]"      "ddr4"      "PIN_G22"       unused       unused       "PIN_G22"      "PIN_G22"     unused        unused        "PIN_G22"     "PIN_G22"     ] \
      [ list "${emif_name}_mem_mem_dq[31]"      "ddr4"      "PIN_J21"       unused       unused       "PIN_J21"      "PIN_J21"     unused        unused        "PIN_J21"     "PIN_J21"     ] \
      [ list "${emif_name}_mem_mem_dq[32]"      "ddr4"      "PIN_D10"       unused       unused         unused       "PIN_A24"     unused        unused        unused        "PIN_A24"     ] \
      [ list "${emif_name}_mem_mem_dq[33]"      "ddr4"      "PIN_F11"       unused       unused         unused       "PIN_D25"     unused        unused        unused        "PIN_D25"     ] \
      [ list "${emif_name}_mem_mem_dq[34]"      "ddr4"      "PIN_E11"       unused       unused         unused       "PIN_C25"     unused        unused        unused        "PIN_C25"     ] \
      [ list "${emif_name}_mem_mem_dq[35]"      "ddr4"      "PIN_G12"       unused       unused         unused       "PIN_D26"     unused        unused        unused        "PIN_D26"     ] \
      [ list "${emif_name}_mem_mem_dq[36]"      "ddr4"      "PIN_C11"       unused       unused         unused       "PIN_E26"     unused        unused        unused        "PIN_E26"     ] \
      [ list "${emif_name}_mem_mem_dq[37]"      "ddr4"      "PIN_F12"       unused       unused         unused       "PIN_C27"     unused        unused        unused        "PIN_C27"     ] \
      [ list "${emif_name}_mem_mem_dq[38]"      "ddr4"      "PIN_C10"       unused       unused         unused       "PIN_C26"     unused        unused        unused        "PIN_C26"     ] \
      [ list "${emif_name}_mem_mem_dq[39]"      "ddr4"      "PIN_D11"       unused       unused         unused       "PIN_F26"     unused        unused        unused        "PIN_F26"     ] \
]

set ashfield_emif_pinout_matrix [ list \
      [ list "NAME"                             "MEM"       "LOC"           "x16_r1"     "x24_r1"     "x32_r1"       "x40_r1"         ] \
      [ list "${emif_name}_pll_ref_clk"         "ddr4"      "PIN_D27"       "PIN_D27"    "PIN_D27"    "PIN_D27"      "PIN_D27"        ] \
      [ list "${emif_name}_oct_oct_rzqin"       "ddr4"      "PIN_J30"       "PIN_J30"    "PIN_J30"    "PIN_J30"      "PIN_J30"        ] \
      [ list "${emif_name}_mem_mem_a[0]"        "ddr4"      "PIN_M28"       "PIN_M28"    "PIN_M28"    "PIN_M28"      "PIN_M28"        ] \
      [ list "${emif_name}_mem_mem_a[1]"        "ddr4"      "PIN_N28"       "PIN_N28"    "PIN_N28"    "PIN_N28"      "PIN_N28"        ] \
      [ list "${emif_name}_mem_mem_a[2]"        "ddr4"      "PIN_R26"       "PIN_R26"    "PIN_R26"    "PIN_R26"      "PIN_R26"        ] \
      [ list "${emif_name}_mem_mem_a[3]"        "ddr4"      "PIN_P26"       "PIN_P26"    "PIN_P26"    "PIN_P26"      "PIN_P26"        ] \
      [ list "${emif_name}_mem_mem_a[4]"        "ddr4"      "PIN_P28"       "PIN_P28"    "PIN_P28"    "PIN_P28"      "PIN_P28"        ] \
      [ list "${emif_name}_mem_mem_a[5]"        "ddr4"      "PIN_R27"       "PIN_R27"    "PIN_R27"    "PIN_R27"      "PIN_R27"        ] \
      [ list "${emif_name}_mem_mem_a[6]"        "ddr4"      "PIN_K26"       "PIN_K26"    "PIN_K26"    "PIN_K26"      "PIN_K26"        ] \
      [ list "${emif_name}_mem_mem_a[7]"        "ddr4"      "PIN_K27"       "PIN_K27"    "PIN_K27"    "PIN_K27"      "PIN_K27"        ] \
      [ list "${emif_name}_mem_mem_a[8]"        "ddr4"      "PIN_N26"       "PIN_N26"    "PIN_N26"    "PIN_N26"      "PIN_N26"        ] \
      [ list "${emif_name}_mem_mem_a[9]"        "ddr4"      "PIN_N27"       "PIN_N27"    "PIN_N27"    "PIN_N27"      "PIN_N27"        ] \
      [ list "${emif_name}_mem_mem_a[10]"       "ddr4"      "PIN_L27"       "PIN_L27"    "PIN_L27"    "PIN_L27"      "PIN_L27"        ] \
      [ list "${emif_name}_mem_mem_a[11]"       "ddr4"      "PIN_M27"       "PIN_M27"    "PIN_M27"    "PIN_M27"      "PIN_M27"        ] \
      [ list "${emif_name}_mem_mem_a[12]"       "ddr4"      "PIN_J29"       "PIN_J29"    "PIN_J29"    "PIN_J29"      "PIN_J29"        ] \
      [ list "${emif_name}_mem_mem_a[13]"       "ddr4"      "PIN_J28"       "PIN_J28"    "PIN_J28"    "PIN_J28"      "PIN_J28"        ] \
      [ list "${emif_name}_mem_mem_a[14]"       "ddr4"      "PIN_K28"       "PIN_K28"    "PIN_K28"    "PIN_K28"      "PIN_K28"        ] \
      [ list "${emif_name}_mem_mem_a[15]"       "ddr4"      "PIN_H31"       "PIN_H31"    "PIN_H31"    "PIN_H31"      "PIN_H31"        ] \
      [ list "${emif_name}_mem_mem_a[16]"       "ddr4"      "PIN_H30"       "PIN_H30"    "PIN_H30"    "PIN_H30"      "PIN_H30"        ] \
      [ list "${emif_name}_mem_mem_ba[0]"       "ddr4"      "PIN_H27"       "PIN_H27"    "PIN_H27"    "PIN_H27"      "PIN_H27"        ] \
      [ list "${emif_name}_mem_mem_ba[1]"       "ddr4"      "PIN_G27"       "PIN_G27"    "PIN_G27"    "PIN_G27"      "PIN_G27"        ] \
      [ list "${emif_name}_mem_mem_bg[0]"       "ddr4"      "PIN_F27"       "PIN_F27"    "PIN_F27"    "PIN_F27"      "PIN_F27"        ] \
      [ list "${emif_name}_mem_mem_act_n"       "ddr4"      "PIN_L29"       "PIN_L29"    "PIN_L29"    "PIN_L29"      "PIN_L29"        ] \
      [ list "${emif_name}_mem_mem_alert_n"     "ddr4"      "PIN_A25"       "PIN_A25"    "PIN_A25"    "PIN_A25"      "PIN_A25"        ] \
      [ list "${emif_name}_mem_mem_ck[0]"       "ddr4"      "PIN_K31"       "PIN_K31"    "PIN_K31"    "PIN_K31"      "PIN_K31"        ] \
      [ list "${emif_name}_mem_mem_ck_n[0]"     "ddr4"      "PIN_L30"       "PIN_L30"    "PIN_L30"    "PIN_L30"      "PIN_L30"        ] \
      [ list "${emif_name}_mem_mem_cke[0]"      "ddr4"      "PIN_M30"       "PIN_M30"    "PIN_M30"    "PIN_M30"      "PIN_M30"        ] \
      [ list "${emif_name}_mem_mem_cs_n[0]"     "ddr4"      "PIN_M29"       "PIN_M29"    "PIN_M29"    "PIN_M29"      "PIN_M29"        ] \
      [ list "${emif_name}_mem_mem_odt[0]"      "ddr4"      "PIN_K30"       "PIN_K30"    "PIN_K30"    "PIN_K30"      "PIN_K30"        ] \
      [ list "${emif_name}_mem_mem_par"         "ddr4"      "PIN_P29"       "PIN_P29"    "PIN_P29"    "PIN_P29"      "PIN_P29"        ] \
      [ list "${emif_name}_mem_mem_reset_n"     "ddr4"      "PIN_P31"       "PIN_P31"    "PIN_P31"    "PIN_P31"      "PIN_P31"        ] \
      [ list "${emif_name}_mem_mem_dqs[0]"      "ddr4"      "PIN_J23"       "PIN_J23"    "PIN_J23"    "PIN_J23"      "PIN_J23"        ] \
      [ list "${emif_name}_mem_mem_dqs[1]"      "ddr4"      "PIN_T23"       "PIN_T23"    "PIN_T23"    "PIN_T23"      "PIN_T23"        ] \
      [ list "${emif_name}_mem_mem_dqs[2]"      "ddr4"      "PIN_N25"       unused       "PIN_N25"    "PIN_N25"      "PIN_N25"        ] \
      [ list "${emif_name}_mem_mem_dqs[3]"      "ddr4"      "PIN_C23"       unused       unused       "PIN_C23"      "PIN_C23"        ] \
      [ list "${emif_name}_mem_mem_dqs[4]"      "ddr4"      "PIN_D8"        unused       unused       unused         "PIN_B24"        ] \
      [ list "${emif_name}_mem_mem_dqs_n[0]"    "ddr4"      "PIN_H23"       "PIN_H23"    "PIN_H23"    "PIN_H23"      "PIN_H23"        ] \
      [ list "${emif_name}_mem_mem_dqs_n[1]"    "ddr4"      "PIN_R24"       "PIN_R24"    "PIN_R24"    "PIN_R24"      "PIN_R24"        ] \
      [ list "${emif_name}_mem_mem_dqs_n[2]"    "ddr4"      "PIN_P25"       unused       "PIN_P25"    "PIN_P25"      "PIN_P25"        ] \
      [ list "${emif_name}_mem_mem_dqs_n[3]"    "ddr4"      "PIN_C22"       unused       unused       "PIN_C22"      "PIN_C22"        ] \
      [ list "${emif_name}_mem_mem_dqs_n[4]"    "ddr4"      "PIN_C8"        unused       unused       unused         "PIN_B25"        ] \
      [ list "${emif_name}_mem_mem_dbi_n[0]"    "ddr4"      "PIN_H22"       "PIN_H22"    "PIN_H22"    "PIN_H22"      "PIN_H22"        ] \
      [ list "${emif_name}_mem_mem_dbi_n[1]"    "ddr4"      "PIN_P24"       "PIN_P24"    "PIN_P24"    "PIN_P24"      "PIN_P24"        ] \
      [ list "${emif_name}_mem_mem_dbi_n[2]"    "ddr4"      "PIN_G24"       unused       "PIN_G24"    "PIN_G24"      "PIN_G24"        ] \
      [ list "${emif_name}_mem_mem_dbi_n[3]"    "ddr4"      "PIN_G23"       unused       unused       "PIN_G23"      "PIN_G23"        ] \
      [ list "${emif_name}_mem_mem_dbi_n[4]"    "ddr4"      "PIN_E12"       unused       unused       unused         "PIN_F25"        ] \
      [ list "${emif_name}_mem_mem_dq[0]"       "ddr4"      "PIN_G22"       "PIN_G22"    "PIN_G22"    "PIN_G22"      "PIN_G22"        ] \
      [ list "${emif_name}_mem_mem_dq[1]"       "ddr4"      "PIN_J21"       "PIN_J21"    "PIN_J21"    "PIN_J21"      "PIN_J21"        ] \
      [ list "${emif_name}_mem_mem_dq[2]"       "ddr4"      "PIN_E22"       "PIN_E22"    "PIN_E22"    "PIN_E22"      "PIN_E22"        ] \
      [ list "${emif_name}_mem_mem_dq[3]"       "ddr4"      "PIN_F22"       "PIN_F22"    "PIN_F22"    "PIN_F22"      "PIN_F22"        ] \
      [ list "${emif_name}_mem_mem_dq[4]"       "ddr4"      "PIN_H21"       "PIN_H21"    "PIN_H21"    "PIN_H21"      "PIN_H21"        ] \
      [ list "${emif_name}_mem_mem_dq[5]"       "ddr4"      "PIN_F21"       "PIN_F21"    "PIN_F21"    "PIN_F21"      "PIN_F21"        ] \
      [ list "${emif_name}_mem_mem_dq[6]"       "ddr4"      "PIN_E21"       "PIN_E21"    "PIN_E21"    "PIN_E21"      "PIN_E21"        ] \
      [ list "${emif_name}_mem_mem_dq[7]"       "ddr4"      "PIN_D21"       "PIN_D21"    "PIN_D21"    "PIN_D21"      "PIN_D21"        ] \
      [ list "${emif_name}_mem_mem_dq[8]"       "ddr4"      "PIN_P23"       "PIN_P23"    "PIN_P23"    "PIN_P23"      "PIN_P23"        ] \
      [ list "${emif_name}_mem_mem_dq[9]"       "ddr4"      "PIN_N22"       "PIN_N22"    "PIN_N22"    "PIN_N22"      "PIN_N22"        ] \
      [ list "${emif_name}_mem_mem_dq[10]"      "ddr4"      "PIN_K23"       "PIN_K23"    "PIN_K23"    "PIN_K23"      "PIN_K23"        ] \
      [ list "${emif_name}_mem_mem_dq[11]"      "ddr4"      "PIN_K22"       "PIN_K22"    "PIN_K22"    "PIN_K22"      "PIN_K22"        ] \
      [ list "${emif_name}_mem_mem_dq[12]"      "ddr4"      "PIN_L22"       "PIN_L22"    "PIN_L22"    "PIN_L22"      "PIN_L22"        ] \
      [ list "${emif_name}_mem_mem_dq[13]"      "ddr4"      "PIN_N23"       "PIN_N23"    "PIN_N23"    "PIN_N23"      "PIN_N23"        ] \
      [ list "${emif_name}_mem_mem_dq[14]"      "ddr4"      "PIN_M22"       "PIN_M22"    "PIN_M22"    "PIN_M22"      "PIN_M22"        ] \
      [ list "${emif_name}_mem_mem_dq[15]"      "ddr4"      "PIN_M23"       "PIN_M23"    "PIN_M23"    "PIN_M23"      "PIN_M23"        ] \
      [ list "${emif_name}_mem_mem_dq[16]"      "ddr4"      "PIN_J25"       unused       "PIN_D26"    "PIN_J25"      "PIN_J25"        ] \
      [ list "${emif_name}_mem_mem_dq[17]"      "ddr4"      "PIN_M25"       unused       "PIN_C27"    "PIN_M25"      "PIN_M25"        ] \
      [ list "${emif_name}_mem_mem_dq[18]"      "ddr4"      "PIN_L25"       unused       "PIN_F26"    "PIN_L25"      "PIN_L25"        ] \
      [ list "${emif_name}_mem_mem_dq[19]"      "ddr4"      "PIN_K24"       unused       "PIN_C26"    "PIN_K24"      "PIN_K24"        ] \
      [ list "${emif_name}_mem_mem_dq[20]"      "ddr4"      "PIN_J24"       unused       "PIN_E26"    "PIN_J24"      "PIN_J24"        ] \
      [ list "${emif_name}_mem_mem_dq[21]"      "ddr4"      "PIN_L24"       unused       "PIN_C25"    "PIN_L24"      "PIN_L24"        ] \
      [ list "${emif_name}_mem_mem_dq[22]"      "ddr4"      "PIN_G25"       unused       "PIN_D25"    "PIN_G25"      "PIN_G25"        ] \
      [ list "${emif_name}_mem_mem_dq[23]"      "ddr4"      "PIN_H26"       unused       "PIN_A24"    "PIN_H26"      "PIN_H26"        ] \
      [ list "${emif_name}_mem_mem_dq[24]"      "ddr4"      "PIN_B22"       unused       unused       "PIN_B22"      "PIN_B22"        ] \
      [ list "${emif_name}_mem_mem_dq[25]"      "ddr4"      "PIN_A21"       unused       unused       "PIN_A21"      "PIN_A21"        ] \
      [ list "${emif_name}_mem_mem_dq[26]"      "ddr4"      "PIN_A22"       unused       unused       "PIN_A22"      "PIN_A22"        ] \
      [ list "${emif_name}_mem_mem_dq[27]"      "ddr4"      "PIN_F24"       unused       unused       "PIN_F24"      "PIN_F24"        ] \
      [ list "${emif_name}_mem_mem_dq[28]"      "ddr4"      "PIN_D23"       unused       unused       "PIN_D23"      "PIN_D23"        ] \
      [ list "${emif_name}_mem_mem_dq[29]"      "ddr4"      "PIN_E23"       unused       unused       "PIN_E23"      "PIN_E23"        ] \
      [ list "${emif_name}_mem_mem_dq[30]"      "ddr4"      "PIN_D24"       unused       unused       "PIN_D24"      "PIN_D24"        ] \
      [ list "${emif_name}_mem_mem_dq[31]"      "ddr4"      "PIN_E24"       unused       unused       "PIN_E24"      "PIN_E24"        ] \
      [ list "${emif_name}_mem_mem_dq[32]"      "ddr4"      "PIN_D10"       unused       unused         unused       "PIN_D26"        ] \
      [ list "${emif_name}_mem_mem_dq[33]"      "ddr4"      "PIN_F11"       unused       unused         unused       "PIN_C27"        ] \
      [ list "${emif_name}_mem_mem_dq[34]"      "ddr4"      "PIN_E11"       unused       unused         unused       "PIN_F26"        ] \
      [ list "${emif_name}_mem_mem_dq[35]"      "ddr4"      "PIN_G12"       unused       unused         unused       "PIN_C26"        ] \
      [ list "${emif_name}_mem_mem_dq[36]"      "ddr4"      "PIN_C11"       unused       unused         unused       "PIN_E26"        ] \
      [ list "${emif_name}_mem_mem_dq[37]"      "ddr4"      "PIN_F12"       unused       unused         unused       "PIN_C25"        ] \
      [ list "${emif_name}_mem_mem_dq[38]"      "ddr4"      "PIN_C10"       unused       unused         unused       "PIN_D25"        ] \
      [ list "${emif_name}_mem_mem_dq[39]"      "ddr4"      "PIN_D11"       unused       unused         unused       "PIN_A24"        ] \
]


if {$board == "devkit"} {
   set pin_matrix $s10soc_emif_pinout_matrix
} elseif {$board == "pe"} {
   set pin_matrix $pe1760c_emif_pinout_matrix
} elseif {$board == "atso12"} {
   set pin_matrix $atso12_emif_pinout_matrix
} elseif {$board == "ashfield"} {
   set pin_matrix $ashfield_emif_pinout_matrix
} else {
   error "Unidentified board; please supply pinout matrix."
}

if {$hps_emif_en} {
   set ranks r1
   set width $hps_emif_width
   set ecc   $hps_emif_ecc_en

   if {$ecc} {
      incr width 8
   }

   set key "x${width}_$ranks"

   # Search for key in the first line
   set key_line [lindex $pin_matrix 0]
   set idx [lsearch $key_line $key]
   
   if {$idx < 0} {
      error "Could not locate configuration $key for EMIF generation"
   }

   set mem_type_idx [lsearch $key_line "MEM"]

   if {$mem_type_idx < 0} {
      error "Could not locate memory type specifier in pinout matrix for EMIF generation"
   }

   puts "key = $key"
   puts "board = $board"
   puts "mem_type = $hps_emif_type"

   # Now add all items
   set skip_first 1
   foreach key_line $pin_matrix {
      if {$skip_first} {
         set skip_first 0
      } else {
         set pin [lindex $key_line $idx]
         set mem_type [lindex $key_line $mem_type_idx]

         if {$pin != "unused" && (($mem_type == $hps_emif_type) || ($mem_type == "both"))} {
            set_location_assignment $pin -to [lindex $key_line 0]
            puts "Setting: set_location_assignment $pin -to [lindex $key_line 0]"
         }
      }
   }
}
